import PropTypes from 'prop-types';
import React__default, { Component, createContext, useContext, useState, createElement } from 'react';
import classNames from 'classnames';
import isEmpty from 'lodash.isempty';
import { createStore } from 'redux';
import remove from 'lodash.remove';
import isEqual from 'lodash.isequal';
import { atom, useAtom } from 'jotai';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

const LOAD_START = 'cueplayer-react/LOAD_START';
const CAN_PLAY = 'cueplayer-react/CAN_PLAY';
const WAITING = 'cueplayer-react/WAITING';
const CAN_PLAY_THROUGH = 'cueplayer-react/CAN_PLAY_THROUGH';
const PLAYING = 'cueplayer-react/PLAYING';
const PLAY = 'cueplayer-react/PLAY';
const PAUSE = 'cueplayer-react/PAUSE';
const END = 'cueplayer-react/END';
const SEEKING = 'cueplayer-react/SEEKING';
const SEEKED = 'cueplayer-react/SEEKED';
const SEEKING_TIME = 'cueplayer-react/SEEKING_TIME';
const END_SEEKING = 'cueplayer-react/END_SEEKING';
const DURATION_CHANGE = 'cueplayer-react/DURATION_CHANGE';
const TIME_UPDATE = 'cueplayer-react/TIME_UPDATE';
const VOLUME_CHANGE = 'cueplayer-react/VOLUME_CHANGE';
const PROGRESS_CHANGE = 'cueplayer-react/PROGRESS_CHANGE';
const RATE_CHANGE = 'cueplayer-react/RATE_CHANGE';
const SUSPEND = 'cueplayer-react/SUSPEND';
const ABORT = 'cueplayer-react/ABORT';
const EMPTIED = 'cueplayer-react/EMPTIED';
const STALLED = 'cueplayer-react/STALLED';
const LOADED_META_DATA = 'cueplayer-react/LOADED_META_DATA';
const LOADED_DATA = 'cueplayer-react/LOADED_DATA';
const RESIZE = 'cueplayer-react/RESIZE';
const ERROR = 'cueplayer-react/ERROR';
const ACTIVATE_TEXT_TRACK = 'cueplayer-react/ACTIVATE_TEXT_TRACK';
const ACTIVATE_METADATA_TRACK = 'cueplayer-react/ACTIVATE_METADATA_TRACK';
const ACTIVATE_METADATA_TRACK_CUE = 'cueplayer-react/ACTIVATE_METADATA_TRACK_CUE';
const DEACTIVATE_METADATA_TRACK_CUE = 'cueplayer-react/DEACTIVATE_METADATA_TRACK_CUE';
function handleLoadStart(videoProps) {
  return {
    type: LOAD_START,
    videoProps
  };
}
function handleCanPlay(videoProps) {
  return {
    type: CAN_PLAY,
    videoProps
  };
}
function handleWaiting(videoProps) {
  return {
    type: WAITING,
    videoProps
  };
}
function handleCanPlayThrough(videoProps) {
  return {
    type: CAN_PLAY_THROUGH,
    videoProps
  };
}
function handlePlaying(videoProps) {
  return {
    type: PLAYING,
    videoProps
  };
}
function handlePlay(videoProps) {
  return {
    type: PLAY,
    videoProps
  };
}
function handlePause(videoProps) {
  return {
    type: PAUSE,
    videoProps
  };
}
function handleEnd(videoProps) {
  return {
    type: END,
    videoProps
  };
}
function handleSeeking(videoProps) {
  return {
    type: SEEKING,
    videoProps
  };
}
function handleSeeked(videoProps) {
  return {
    type: SEEKED,
    videoProps
  };
}
function handleDurationChange(videoProps) {
  return {
    type: DURATION_CHANGE,
    videoProps
  };
}
function handleTimeUpdate(videoProps) {
  return {
    type: TIME_UPDATE,
    videoProps
  };
}
function handleVolumeChange(videoProps) {
  return {
    type: VOLUME_CHANGE,
    videoProps
  };
}
function handleProgressChange(videoProps) {
  return {
    type: PROGRESS_CHANGE,
    videoProps
  };
}
function handleRateChange(videoProps) {
  return {
    type: RATE_CHANGE,
    videoProps
  };
}
function handleSuspend(videoProps) {
  return {
    type: SUSPEND,
    videoProps
  };
}
function handleAbort(videoProps) {
  return {
    type: ABORT,
    videoProps
  };
}
function handleEmptied(videoProps) {
  return {
    type: EMPTIED,
    videoProps
  };
}
function handleStalled(videoProps) {
  return {
    type: STALLED,
    videoProps
  };
}
function handleLoadedMetaData(videoProps) {
  return {
    type: LOADED_META_DATA,
    videoProps
  };
}
function handleLoadedData(videoProps) {
  return {
    type: LOADED_DATA,
    videoProps
  };
}
function handleResize(videoProps) {
  return {
    type: RESIZE,
    videoProps
  };
}
function handleError(videoProps) {
  return {
    type: ERROR,
    videoProps
  };
}
function handleSeekingTime(time) {
  return {
    type: SEEKING_TIME,
    time
  };
}
function handleEndSeeking(time) {
  return {
    type: END_SEEKING,
    time
  };
}
function activateTextTrack(textTrack) {
  return {
    type: ACTIVATE_TEXT_TRACK,
    textTrack
  };
}
function activateMetadataTrack(metadataTrack) {
  return {
    type: ACTIVATE_METADATA_TRACK,
    metadataTrack
  };
}
function activateMetadataTrackCue(cue) {
  return {
    type: ACTIVATE_METADATA_TRACK_CUE,
    cue
  };
}
function deactivateMetadataTrackCue(cue) {
  return {
    type: DEACTIVATE_METADATA_TRACK_CUE,
    cue
  };
}

var videoActions = /*#__PURE__*/Object.freeze({
  __proto__: null,
  LOAD_START: LOAD_START,
  CAN_PLAY: CAN_PLAY,
  WAITING: WAITING,
  CAN_PLAY_THROUGH: CAN_PLAY_THROUGH,
  PLAYING: PLAYING,
  PLAY: PLAY,
  PAUSE: PAUSE,
  END: END,
  SEEKING: SEEKING,
  SEEKED: SEEKED,
  SEEKING_TIME: SEEKING_TIME,
  END_SEEKING: END_SEEKING,
  DURATION_CHANGE: DURATION_CHANGE,
  TIME_UPDATE: TIME_UPDATE,
  VOLUME_CHANGE: VOLUME_CHANGE,
  PROGRESS_CHANGE: PROGRESS_CHANGE,
  RATE_CHANGE: RATE_CHANGE,
  SUSPEND: SUSPEND,
  ABORT: ABORT,
  EMPTIED: EMPTIED,
  STALLED: STALLED,
  LOADED_META_DATA: LOADED_META_DATA,
  LOADED_DATA: LOADED_DATA,
  RESIZE: RESIZE,
  ERROR: ERROR,
  ACTIVATE_TEXT_TRACK: ACTIVATE_TEXT_TRACK,
  ACTIVATE_METADATA_TRACK: ACTIVATE_METADATA_TRACK,
  ACTIVATE_METADATA_TRACK_CUE: ACTIVATE_METADATA_TRACK_CUE,
  DEACTIVATE_METADATA_TRACK_CUE: DEACTIVATE_METADATA_TRACK_CUE,
  handleLoadStart: handleLoadStart,
  handleCanPlay: handleCanPlay,
  handleWaiting: handleWaiting,
  handleCanPlayThrough: handleCanPlayThrough,
  handlePlaying: handlePlaying,
  handlePlay: handlePlay,
  handlePause: handlePause,
  handleEnd: handleEnd,
  handleSeeking: handleSeeking,
  handleSeeked: handleSeeked,
  handleDurationChange: handleDurationChange,
  handleTimeUpdate: handleTimeUpdate,
  handleVolumeChange: handleVolumeChange,
  handleProgressChange: handleProgressChange,
  handleRateChange: handleRateChange,
  handleSuspend: handleSuspend,
  handleAbort: handleAbort,
  handleEmptied: handleEmptied,
  handleStalled: handleStalled,
  handleLoadedMetaData: handleLoadedMetaData,
  handleLoadedData: handleLoadedData,
  handleResize: handleResize,
  handleError: handleError,
  handleSeekingTime: handleSeekingTime,
  handleEndSeeking: handleEndSeeking,
  activateTextTrack: activateTextTrack,
  activateMetadataTrack: activateMetadataTrack,
  activateMetadataTrackCue: activateMetadataTrackCue,
  deactivateMetadataTrackCue: deactivateMetadataTrackCue
});

class Fullscreen {
  request(elm) {
    if (elm.requestFullscreen) {
      elm.requestFullscreen();
    } else if (elm.webkitRequestFullscreen) {
      elm.webkitRequestFullscreen();
    } else if (elm.mozRequestFullScreen) {
      elm.mozRequestFullScreen();
    } else if (elm.msRequestFullscreen) {
      elm.msRequestFullscreen();
    }
  }

  exit() {
    if (document.exitFullscreen) {
      document.exitFullscreen();
    } else if (document.webkitExitFullscreen) {
      document.webkitExitFullscreen();
    } else if (document.mozCancelFullScreen) {
      document.mozCancelFullScreen();
    } else if (document.msExitFullscreen) {
      document.msExitFullscreen();
    }
  }

  get isFullscreen() {
    return document.fullscreenElement || document.webkitFullscreenElement || document.mozFullScreenElement || document.msFullscreenElement;
  }

  get enabled() {
    return document.fullscreenEnabled || document.webkitFullscreenEnabled || document.mozFullScreenEnabled || document.msFullscreenEnabled;
  }

  addEventListener(handler) {
    document.addEventListener('fullscreenchange', handler);
    document.addEventListener('webkitfullscreenchange', handler);
    document.addEventListener('mozfullscreenchange', handler);
    document.addEventListener('MSFullscreenChange', handler);
  }

  removeEventListener(handler) {
    document.removeEventListener('fullscreenchange', handler);
    document.removeEventListener('webkitfullscreenchange', handler);
    document.removeEventListener('mozfullscreenchange', handler);
    document.removeEventListener('MSFullscreenChange', handler);
  }

}

var fullscreen = new Fullscreen();

const OPERATE = 'cueplayer-react/OPERATE';
const FULLSCREEN_CHANGE = 'cueplayer-react/FULLSCREEN_CHANGE';
const PLAYER_ACTIVATE = 'cueplayer-react/PLAYER_ACTIVATE';
const USER_ACTIVATE = 'cueplayer-react/USER_ACTIVATE';
function handleFullscreenChange(isFullscreen) {
  return {
    type: FULLSCREEN_CHANGE,
    isFullscreen
  };
}
function activate(activity) {
  return {
    type: PLAYER_ACTIVATE,
    activity
  };
}
function userActivate(activity) {
  return {
    type: USER_ACTIVATE,
    activity
  };
}
function play(operation) {
  if (operation === void 0) {
    operation = {
      action: 'play',
      source: ''
    };
  }

  this.video.play();
  return {
    type: OPERATE,
    operation
  };
}
function pause(operation) {
  if (operation === void 0) {
    operation = {
      action: 'pause',
      source: ''
    };
  }

  this.video.pause();
  return {
    type: OPERATE,
    operation
  };
}
function togglePlay(operation) {
  if (operation === void 0) {
    operation = {
      action: 'toggle-play',
      source: ''
    };
  }

  this.video.togglePlay();
  return {
    type: OPERATE,
    operation
  };
} // seek video by time

function seek(time, operation) {
  if (operation === void 0) {
    operation = {
      action: 'seek',
      source: ''
    };
  }

  this.video.seek(time);
  return {
    type: OPERATE,
    operation
  };
} // jump forward x seconds

function forward(seconds, operation) {
  if (operation === void 0) {
    operation = {
      action: "forward-" + seconds,
      source: ''
    };
  }

  this.video.forward(seconds);
  return {
    type: OPERATE,
    operation
  };
} // jump back x seconds

function replay(seconds, operation) {
  if (operation === void 0) {
    operation = {
      action: "replay-" + seconds,
      source: ''
    };
  }

  this.video.replay(seconds);
  return {
    type: OPERATE,
    operation
  };
}
function changeRate(rate, operation) {
  if (operation === void 0) {
    operation = {
      action: 'change-rate',
      source: ''
    };
  }

  this.video.playbackRate = rate;
  return {
    type: OPERATE,
    operation
  };
}
function changeVolume(volume, operation) {
  if (operation === void 0) {
    operation = {
      action: 'change-volume',
      source: ''
    };
  }

  let v = volume;

  if (volume < 0) {
    v = 0;
  }

  if (volume > 1) {
    v = 1;
  }

  this.video.volume = v;
  return {
    type: OPERATE,
    operation
  };
}
function mute(muted, operation) {
  if (operation === void 0) {
    operation = {
      action: muted ? 'muted' : 'unmuted',
      source: ''
    };
  }

  this.video.muted = muted;
  return {
    type: OPERATE,
    operation
  };
}
function toggleFullscreen(player, fullscreenElement) {
  if (fullscreen.enabled) {
    const element = fullscreenElement || this.rootElement;

    if (fullscreen.isFullscreen) {
      fullscreen.exit();
    } else {
      fullscreen.request(element);
    }

    return {
      type: OPERATE,
      operation: {
        action: 'toggle-fullscreen',
        source: ''
      }
    };
  }

  return {
    type: FULLSCREEN_CHANGE,
    isFullscreen: !player.isFullscreen
  };
}

var playerActions = /*#__PURE__*/Object.freeze({
  __proto__: null,
  OPERATE: OPERATE,
  FULLSCREEN_CHANGE: FULLSCREEN_CHANGE,
  PLAYER_ACTIVATE: PLAYER_ACTIVATE,
  USER_ACTIVATE: USER_ACTIVATE,
  handleFullscreenChange: handleFullscreenChange,
  activate: activate,
  userActivate: userActivate,
  play: play,
  pause: pause,
  togglePlay: togglePlay,
  seek: seek,
  forward: forward,
  replay: replay,
  changeRate: changeRate,
  changeVolume: changeVolume,
  mute: mute,
  toggleFullscreen: toggleFullscreen
});

const initialState = {
  currentSrc: null,
  duration: 0,
  currentTime: 0,
  seekingTime: 0,
  buffered: null,
  waiting: false,
  seeking: false,
  paused: true,
  autoPaused: false,
  ended: false,
  playbackRate: 1,
  muted: false,
  volume: 1,
  readyState: 0,
  networkState: 0,
  videoWidth: 0,
  videoHeight: 0,
  hasStarted: false,
  userActivity: true,
  isActive: false,
  isFullscreen: false,
  activeTextTrack: null,
  activeMetadataTracks: [],
  activeMetadataTrackCues: []
};
function player(state, action) {
  if (state === void 0) {
    state = initialState;
  }

  switch (action.type) {
    case USER_ACTIVATE:
      return _extends({}, state, {
        userActivity: action.activity
      });

    case PLAYER_ACTIVATE:
      return _extends({}, state, {
        isActive: action.activity
      });

    case FULLSCREEN_CHANGE:
      return _extends({}, state, {
        isFullscreen: !!action.isFullscreen
      });

    case SEEKING_TIME:
      return _extends({}, state, {
        seekingTime: action.time
      });

    case END_SEEKING:
      return _extends({}, state, {
        seekingTime: 0
      });

    case LOAD_START:
      return _extends({}, state, action.videoProps, {
        hasStarted: false,
        ended: false
      });

    case CAN_PLAY:
      return _extends({}, state, action.videoProps, {
        waiting: false
      });

    case WAITING:
      return _extends({}, state, action.videoProps, {
        waiting: true
      });

    case CAN_PLAY_THROUGH:
    case PLAYING:
      return _extends({}, state, action.videoProps, {
        waiting: false
      });

    case PLAY:
      return _extends({}, state, action.videoProps, {
        ended: false,
        paused: false,
        autoPaused: false,
        waiting: false,
        hasStarted: true
      });

    case PAUSE:
      return _extends({}, state, action.videoProps, {
        paused: true
      });

    case END:
      return _extends({}, state, action.videoProps, {
        ended: true
      });

    case SEEKING:
      return _extends({}, state, action.videoProps, {
        seeking: true
      });

    case SEEKED:
      return _extends({}, state, action.videoProps, {
        seeking: false
      });

    case ERROR:
      return _extends({}, state, action.videoProps, {
        error: 'UNKNOWN ERROR',
        ended: true
      });

    case DURATION_CHANGE:
    case TIME_UPDATE:
    case VOLUME_CHANGE:
    case PROGRESS_CHANGE:
    case RATE_CHANGE:
    case SUSPEND:
    case ABORT:
    case EMPTIED:
    case STALLED:
    case LOADED_META_DATA:
    case LOADED_DATA:
    case RESIZE:
      return _extends({}, state, action.videoProps);

    case ACTIVATE_TEXT_TRACK:
      return _extends({}, state, {
        activeTextTrack: action.textTrack
      });

    case ACTIVATE_METADATA_TRACK:
      action.metadataTrack.mode = 'showing';
      return _extends({}, state, {
        activeMetadataTracks: [...state.activeMetadataTracks, action.metadataTrack]
      });

    case ACTIVATE_METADATA_TRACK_CUE:
      return _extends({}, state, {
        activeMetadataTrackCues: [action.cue]
      });

    case DEACTIVATE_METADATA_TRACK_CUE:
      remove(state.activeMetadataTrackCues, cue => {
        return isEqual(action.cue, cue);
      });
      return _extends({}, state, {
        activeMetadataTrackCues: [...state.activeMetadataTrackCues]
      });

    default:
      return state;
  }
}

const initialState$1 = {
  count: 0,
  operation: {
    action: '',
    source: ''
  }
};
function operation(state, action) {
  if (state === void 0) {
    state = initialState$1;
  }

  switch (action.type) {
    case OPERATE:
      return _extends({}, state, {
        count: state.count + 1,
        operation: _extends({}, state.operation, action.operation)
      });

    default:
      return state;
  }
}

function reducer (state, action) {
  if (state === void 0) {
    state = {};
  }

  return {
    player: player(state.player, action),
    operation: operation(state.operation, action)
  };
}
const playerReducer = player;
const operationReducer = operation;

class Manager {
  constructor(store) {
    this.store = store || createStore(reducer);
    this.video = null;
    this.rootElement = null;
  }

  getActions() {
    const manager = this;
    const {
      dispatch
    } = this.store;

    const actions = _extends({}, playerActions, videoActions);

    function bindActionCreator(actionCreator) {
      return function bindAction() {
        // eslint-disable-next-line prefer-rest-params
        const action = actionCreator.apply(manager, arguments);

        if (typeof action !== 'undefined') {
          dispatch(action);
        }
      };
    }

    return Object.keys(actions).filter(key => typeof actions[key] === 'function').reduce((boundActions, key) => {
      boundActions[key] = bindActionCreator(actions[key]);
      return boundActions;
    }, {});
  }

  getState() {
    return this.store.getState();
  } // subscribe state change


  subscribeToStateChange(listener, getState) {
    if (!getState) {
      getState = this.getState.bind(this);
    }

    let prevState = getState();

    const handleChange = () => {
      const state = getState();

      if (state === prevState) {
        return;
      }

      const prevStateCopy = prevState;
      prevState = state;
      listener(state, prevStateCopy);
    };

    return this.store.subscribe(handleChange);
  } // subscribe to operation state change


  subscribeToOperationStateChange(listener) {
    return this.subscribeToStateChange(listener, () => this.getState().operation);
  } // subscribe to player state change


  subscribeToPlayerStateChange(listener) {
    return this.subscribeToStateChange(listener, () => this.getState().player);
  }

}

const propTypes = {
  actions: PropTypes.object,
  player: PropTypes.object,
  position: PropTypes.string,
  className: PropTypes.string
};
const defaultProps = {
  position: 'center'
};
class BigPlayButton extends Component {
  constructor(props) {
    super(props);
    this.handleClick = this.handleClick.bind(this);
  }

  componentDidMount() {}

  handleClick() {
    const {
      actions
    } = this.props;
    actions.play();
  }

  render() {
    const {
      player,
      position
    } = this.props;
    return /*#__PURE__*/React__default.createElement("button", {
      className: classNames('cueplayer-react-button', 'cueplayer-react-big-play-button', "cueplayer-react-big-play-button-" + position, this.props.className, {
        'big-play-button-hide': player.hasStarted || !player.currentSrc
      }),
      type: "button",
      "aria-live": "polite",
      tabIndex: "0",
      onClick: this.handleClick
    }, /*#__PURE__*/React__default.createElement("span", {
      className: "cueplayer-react-control-text"
    }, "Play Video"));
  }

}
BigPlayButton.propTypes = propTypes;
BigPlayButton.defaultProps = defaultProps;
BigPlayButton.displayName = 'BigPlayButton';

const propTypes$1 = {
  player: PropTypes.object,
  className: PropTypes.string
};
function LoadingSpinner(_ref) {
  let {
    player,
    className
  } = _ref;

  if (player.error) {
    return null;
  }

  return /*#__PURE__*/React__default.createElement("div", {
    className: classNames('cueplayer-react-loading-spinner', className)
  });
}
LoadingSpinner.propTypes = propTypes$1;
LoadingSpinner.displayName = 'LoadingSpinner';

const propTypes$2 = {
  poster: PropTypes.string,
  player: PropTypes.object,
  actions: PropTypes.object,
  className: PropTypes.string
};

function PosterImage(_ref) {
  let {
    poster,
    player,
    actions,
    className
  } = _ref;

  if (!poster || player.hasStarted) {
    return null;
  }

  return /*#__PURE__*/React__default.createElement("div", {
    className: classNames('cueplayer-react-poster', className),
    style: {
      backgroundImage: "url(\"" + poster + "\")"
    },
    onClick: () => {
      if (player.paused) {
        actions.play();
      }
    }
  });
}

PosterImage.propTypes = propTypes$2;
PosterImage.displayName = 'PosterImage';

// eslint-disable-next-line no-self-compare

const isNaN = Number.isNaN || (value => value !== value);
/**
 * @file format-time.js
 *
 * Format seconds as a time string, H:MM:SS or M:SS
 * Supplying a guide (in seconds) will force a number of leading zeros
 * to cover the length of the guide
 *
 * @param  {Number} seconds Number of seconds to be turned into a string
 * @param  {Number} guide   Number (in seconds) to model the string after
 * @return {String}         Time formatted as H:MM:SS or M:SS
 * @private
 * @function formatTime
 */


function formatTime(seconds, guide) {
  if (seconds === void 0) {
    seconds = 0;
  }

  if (guide === void 0) {
    guide = seconds;
  }

  let s = Math.floor(seconds % 60);
  let m = Math.floor(seconds / 60 % 60);
  let h = Math.floor(seconds / 3600);
  const gm = Math.floor(guide / 60 % 60);
  const gh = Math.floor(guide / 3600); // handle invalid times

  if (isNaN(seconds) || seconds === Infinity) {
    // '-' is false for all relational operators (e.g. <, >=) so this setting
    // will add the minimum number of fields specified by the guide
    h = '-';
    m = '-';
    s = '-';
  } // Check if we need to show hours


  h = h > 0 || gh > 0 ? h + ":" : ''; // If hours are showing, we may need to add a leading zero.
  // Always show at least one digit of minutes.

  m = ((h || gm >= 10) && m < 10 ? "0" + m : m) + ":"; // Check if leading zero is need for seconds

  s = s < 10 ? "0" + s : s;
  return h + m + s;
} // Check if the element belongs to a video element
// only accept <source />, <track />,
// <MyComponent isVideoChild />
// elements

function isVideoChild(c) {
  if (c.props && c.props.isVideoChild) {
    return true;
  }

  return c.type === 'source' || c.type === 'track';
}

const find = (elements, func) => elements.filter(func)[0]; // check if two components are the same type


const isTypeEqual = (component1, component2) => {
  const type1 = component1.type;
  const type2 = component2.type;

  if (typeof type1 === 'string' || typeof type2 === 'string') {
    return type1 === type2;
  }

  if (typeof type1 === 'function' && typeof type2 === 'function') {
    return type1.displayName === type2.displayName;
  }

  return false;
}; // merge default children
// sort them by `order` property
// filter them by `disabled` property


function mergeAndSortChildren(defaultChildren, _children, _parentProps, defaultOrder) {
  if (defaultOrder === void 0) {
    defaultOrder = 1;
  }

  const children = React__default.Children.toArray(_children);

  const parentProps = _objectWithoutPropertiesLoose(_parentProps, ["order"]); // ignore order from parent


  return children.filter(e => !e.props.disabled) // filter the disabled components
  .concat(defaultChildren.filter(c => !find(children, component => isTypeEqual(component, c)))).map(element => {
    const defaultComponent = find(defaultChildren, c => isTypeEqual(c, element));
    const defaultProps = defaultComponent ? defaultComponent.props : {};

    const props = _extends({}, parentProps, defaultProps, element.props);

    const e = /*#__PURE__*/React__default.cloneElement(element, props, element.props.children);
    return e;
  }).sort((a, b) => (a.props.order || defaultOrder) - (b.props.order || defaultOrder));
}
/**
 * Temporary utility for generating the warnings
 */

function deprecatedWarning(oldMethodCall, newMethodCall) {
  // eslint-disable-next-line no-console
  console.warn("WARNING: " + oldMethodCall + " will be deprecated soon! Please use " + newMethodCall + " instead.");
}
function throttle(callback, limit) {
  let wait = false;
  return () => {
    if (!wait) {
      // eslint-disable-next-line prefer-rest-params
      callback(...arguments);
      wait = true;
      setTimeout(() => {
        wait = false;
      }, limit);
    }
  };
}
const mediaProperties = ['error', 'src', 'srcObject', 'currentSrc', 'crossOrigin', 'networkState', 'preload', 'buffered', 'readyState', 'seeking', 'currentTime', 'duration', 'paused', 'defaultPlaybackRate', 'playbackRate', 'played', 'seekable', 'ended', 'autoplay', 'loop', 'mediaGroup', 'controller', 'controls', 'volume', 'muted', 'defaultMuted', 'audioTracks', 'videoTracks', 'textTracks', 'width', 'height', 'videoWidth', 'videoHeight', 'poster'];

const propTypes$3 = {
  actions: PropTypes.object,
  player: PropTypes.object,
  children: PropTypes.any,
  startTime: PropTypes.number,
  loop: PropTypes.bool,
  muted: PropTypes.bool,
  autoPlay: PropTypes.bool,
  playsInline: PropTypes.bool,
  src: PropTypes.string,
  poster: PropTypes.string,
  className: PropTypes.string,
  preload: PropTypes.oneOf(['auto', 'metadata', 'none']),
  crossOrigin: PropTypes.string,
  onLoadStart: PropTypes.func,
  onWaiting: PropTypes.func,
  onCanPlay: PropTypes.func,
  onCanPlayThrough: PropTypes.func,
  onPlaying: PropTypes.func,
  onEnded: PropTypes.func,
  onSeeking: PropTypes.func,
  onSeeked: PropTypes.func,
  onPlay: PropTypes.func,
  onPause: PropTypes.func,
  onProgress: PropTypes.func,
  onDurationChange: PropTypes.func,
  onError: PropTypes.func,
  onSuspend: PropTypes.func,
  onAbort: PropTypes.func,
  onEmptied: PropTypes.func,
  onStalled: PropTypes.func,
  onLoadedMetadata: PropTypes.func,
  onLoadedData: PropTypes.func,
  onTimeUpdate: PropTypes.func,
  onRateChange: PropTypes.func,
  onVolumeChange: PropTypes.func,
  onResize: PropTypes.func
};
class Video extends Component {
  constructor(props) {
    super(props);
    this.video = null; // the html5 video

    this.play = this.play.bind(this);
    this.pause = this.pause.bind(this);
    this.seek = this.seek.bind(this);
    this.forward = this.forward.bind(this);
    this.replay = this.replay.bind(this);
    this.addTextTrack = this.addTextTrack.bind(this);
    this.toggleFullscreen = this.toggleFullscreen.bind(this);
    this.getProperties = this.getProperties.bind(this);
    this.renderChildren = this.renderChildren.bind(this);
    this.handleLoadStart = this.handleLoadStart.bind(this);
    this.handleCanPlay = this.handleCanPlay.bind(this);
    this.handleCanPlayThrough = this.handleCanPlayThrough.bind(this);
    this.handlePlay = this.handlePlay.bind(this);
    this.handlePlaying = this.handlePlaying.bind(this);
    this.handlePause = this.handlePause.bind(this);
    this.handleEnded = this.handleEnded.bind(this);
    this.handleWaiting = this.handleWaiting.bind(this);
    this.handleSeeking = this.handleSeeking.bind(this);
    this.handleSeeked = this.handleSeeked.bind(this);
    this.handleFullscreenChange = this.handleFullscreenChange.bind(this);
    this.handleError = this.handleError.bind(this);
    this.handleSuspend = this.handleSuspend.bind(this);
    this.handleAbort = this.handleAbort.bind(this);
    this.handleEmptied = this.handleEmptied.bind(this);
    this.handleStalled = this.handleStalled.bind(this);
    this.handleLoadedMetaData = this.handleLoadedMetaData.bind(this);
    this.handleLoadedData = this.handleLoadedData.bind(this);
    this.handleTimeUpdate = this.handleTimeUpdate.bind(this);
    this.handleRateChange = this.handleRateChange.bind(this);
    this.handleVolumeChange = this.handleVolumeChange.bind(this);
    this.handleDurationChange = this.handleDurationChange.bind(this);
    this.handleProgress = throttle(this.handleProgress.bind(this), 250);
    this.handleKeypress = this.handleKeypress.bind(this);
    this.handleTextTrackChange = this.handleTextTrackChange.bind(this);
  }

  componentDidMount() {
    this.forceUpdate(); // make sure the children can get the video property

    if (this.video && this.video.textTracks) {
      this.video.textTracks.onaddtrack = this.handleTextTrackChange;
      this.video.textTracks.onremovetrack = this.handleTextTrackChange;
    }
  } // get all video properties


  getProperties() {
    if (!this.video) {
      return null;
    }

    return mediaProperties.reduce((properties, key) => {
      properties[key] = this.video[key];
      return properties;
    }, {});
  } // get playback rate


  get playbackRate() {
    return this.video.playbackRate;
  } // set playback rate
  // speed of video


  set playbackRate(rate) {
    this.video.playbackRate = rate;
  }

  get muted() {
    return this.video.muted;
  }

  set muted(val) {
    this.video.muted = val;
  }

  get volume() {
    return this.video.volume;
  }

  set volume(val) {
    if (val > 1) {
      val = 1;
    }

    if (val < 0) {
      val = 0;
    }

    this.video.volume = val;
  } // video width


  get videoWidth() {
    return this.video.videoWidth;
  } // video height


  get videoHeight() {
    return this.video.videoHeight;
  }

  handleTextTrackChange() {
    const {
      actions,
      player
    } = this.props;

    if (this.video && this.video.textTracks) {
      const activeTextTrack = Array.from(this.video.textTracks).find(textTrack => textTrack.mode === 'showing');

      if (activeTextTrack !== player.activeTextTrack) {
        actions.activateTextTrack(activeTextTrack);
      }
    }
  } // play the video


  play() {
    const promise = this.video.play();

    if (promise !== undefined) {
      promise.catch(() => {}).then(() => {});
    }
  } // pause the video


  pause() {
    const promise = this.video.pause();

    if (promise !== undefined) {
      promise.catch(() => {}).then(() => {});
    }
  } // Change the video source and re-load the video:


  load() {
    this.video.load();
  } // Add a new text track to the video


  addTextTrack() {
    this.video.addTextTrack(...arguments);
  } // Check if your browser can play different types of video:


  canPlayType() {
    this.video.canPlayType(...arguments);
  } // toggle play


  togglePlay() {
    if (this.video.paused) {
      this.play();
    } else {
      this.pause();
    }
  } // seek video by time


  seek(time) {
    try {
      this.video.currentTime = time;
    } catch (e) {// console.log(e, 'Video is not ready.')
    }
  } // jump forward x seconds


  forward(seconds) {
    this.seek(this.video.currentTime + seconds);
  } // jump back x seconds


  replay(seconds) {
    this.forward(-seconds);
  } // enter or exist full screen


  toggleFullscreen() {
    const {
      player,
      actions
    } = this.props;
    actions.toggleFullscreen(player);
  } // Fired when the user agent
  // begins looking for media data


  handleLoadStart() {
    const {
      actions,
      onLoadStart
    } = this.props;
    actions.handleLoadStart(this.getProperties());

    if (onLoadStart) {
      onLoadStart(...arguments);
    }
  } // A handler for events that
  // signal that waiting has ended


  handleCanPlay() {
    const {
      actions,
      onCanPlay
    } = this.props;
    actions.handleCanPlay(this.getProperties());

    if (onCanPlay) {
      onCanPlay(...arguments);
    }
  } // A handler for events that
  // signal that waiting has ended


  handleCanPlayThrough() {
    const {
      actions,
      onCanPlayThrough
    } = this.props;
    actions.handleCanPlayThrough(this.getProperties());

    if (onCanPlayThrough) {
      onCanPlayThrough(...arguments);
    }
  } // A handler for events that
  // signal that waiting has ended


  handlePlaying() {
    const {
      actions,
      onPlaying
    } = this.props;
    actions.handlePlaying(this.getProperties());

    if (onPlaying) {
      onPlaying(...arguments);
    }
  } // Fired whenever the media has been started


  handlePlay() {
    const {
      actions,
      onPlay
    } = this.props;
    actions.handlePlay(this.getProperties());

    if (onPlay) {
      onPlay(...arguments);
    }
  } // Fired whenever the media has been paused


  handlePause() {
    const {
      actions,
      onPause
    } = this.props;
    actions.handlePause(this.getProperties());

    if (onPause) {
      onPause(...arguments);
    }
  } // Fired when the duration of
  // the media resource is first known or changed


  handleDurationChange() {
    const {
      actions,
      onDurationChange
    } = this.props;
    actions.handleDurationChange(this.getProperties());

    if (onDurationChange) {
      onDurationChange(...arguments);
    }
  } // Fired while the user agent
  // is downloading media data


  handleProgress() {
    const {
      actions,
      onProgress
    } = this.props;

    if (this.video) {
      actions.handleProgressChange(this.getProperties());
    }

    if (onProgress) {
      onProgress(...arguments);
    }
  } // Fired when the end of the media resource
  // is reached (currentTime == duration)


  handleEnded() {
    const {
      loop,
      player,
      actions,
      onEnded
    } = this.props;

    if (loop) {
      this.seek(0);
      this.play();
    } else if (!player.paused) {
      this.pause();
    }

    actions.handleEnd(this.getProperties());

    if (onEnded) {
      onEnded(...arguments);
    }
  } // Fired whenever the media begins waiting


  handleWaiting() {
    const {
      actions,
      onWaiting
    } = this.props;
    actions.handleWaiting(this.getProperties());

    if (onWaiting) {
      onWaiting(...arguments);
    }
  } // Fired whenever the player
  // is jumping to a new time


  handleSeeking() {
    const {
      actions,
      onSeeking
    } = this.props;
    actions.handleSeeking(this.getProperties());

    if (onSeeking) {
      onSeeking(...arguments);
    }
  } // Fired when the player has
  // finished jumping to a new time


  handleSeeked() {
    const {
      actions,
      onSeeked
    } = this.props;
    actions.handleSeeked(this.getProperties());

    if (onSeeked) {
      onSeeked(...arguments);
    }
  } // Handle Fullscreen Change


  handleFullscreenChange() {} // Fires when the browser is
  // intentionally not getting media data


  handleSuspend() {
    const {
      actions,
      onSuspend
    } = this.props;
    actions.handleSuspend(this.getProperties());

    if (onSuspend) {
      onSuspend(...arguments);
    }
  } // Fires when the loading of an audio/video is aborted


  handleAbort() {
    const {
      actions,
      onAbort
    } = this.props;
    actions.handleAbort(this.getProperties());

    if (onAbort) {
      onAbort(...arguments);
    }
  } // Fires when the current playlist is empty


  handleEmptied() {
    const {
      actions,
      onEmptied
    } = this.props;
    actions.handleEmptied(this.getProperties());

    if (onEmptied) {
      onEmptied(...arguments);
    }
  } // Fires when the browser is trying to
  // get media data, but data is not available


  handleStalled() {
    const {
      actions,
      onStalled
    } = this.props;
    actions.handleStalled(this.getProperties());

    if (onStalled) {
      onStalled(...arguments);
    }
  } // Fires when the browser has loaded
  // meta data for the audio/video


  handleLoadedMetaData() {
    const {
      actions,
      onLoadedMetadata,
      startTime
    } = this.props;

    if (startTime && startTime > 0) {
      this.video.currentTime = startTime;
    }

    actions.handleLoadedMetaData(this.getProperties());

    if (onLoadedMetadata) {
      onLoadedMetadata(...arguments);
    }
  } // Fires when the browser has loaded
  // the current frame of the audio/video


  handleLoadedData() {
    const {
      actions,
      onLoadedData
    } = this.props;
    actions.handleLoadedData(this.getProperties());

    if (onLoadedData) {
      onLoadedData(...arguments);
    }
  } // Fires when the current
  // playback position has changed


  handleTimeUpdate() {
    const {
      actions,
      onTimeUpdate
    } = this.props;
    actions.handleTimeUpdate(this.getProperties());

    if (onTimeUpdate) {
      onTimeUpdate(...arguments);
    }
  }
  /**
   * Fires when the playing speed of the audio/video is changed
   */


  handleRateChange() {
    const {
      actions,
      onRateChange
    } = this.props;
    actions.handleRateChange(this.getProperties());

    if (onRateChange) {
      onRateChange(...arguments);
    }
  } // Fires when the volume has been changed


  handleVolumeChange() {
    const {
      actions,
      onVolumeChange
    } = this.props;
    actions.handleVolumeChange(this.getProperties());

    if (onVolumeChange) {
      onVolumeChange(...arguments);
    }
  } // Fires when an error occurred
  // during the loading of an audio/video


  handleError() {
    const {
      actions,
      onError
    } = this.props;
    actions.handleError(this.getProperties());

    if (onError) {
      onError(...arguments);
    }
  }

  handleResize() {
    const {
      actions,
      onResize
    } = this.props;
    actions.handleResize(this.getProperties());

    if (onResize) {
      onResize(...arguments);
    }
  }

  handleKeypress() {}

  renderChildren() {
    var _this = this;

    const props = _extends({}, this.props, {
      video: this.video
    }); // to make sure the children can get video property


    if (!this.video) {
      return null;
    } // only keep <source />, <track />, <MyComponent isVideoChild /> elements


    return React__default.Children.toArray(this.props.children).filter(isVideoChild).map(c => {
      let cprops;

      if (typeof c.type === 'string') {
        // add onError to <source />
        if (c.type === 'source') {
          cprops = _extends({}, c.props);
          const preOnError = cprops.onError;

          cprops.onError = function () {
            if (preOnError) {
              preOnError(...arguments);
            }

            _this.handleError(...arguments);
          };
        }
      } else {
        cprops = props;
      }

      return /*#__PURE__*/React__default.cloneElement(c, cprops);
    });
  }

  render() {
    const {
      loop,
      poster,
      preload,
      src,
      autoPlay,
      playsInline,
      muted,
      crossOrigin,
      videoId
    } = this.props;
    return /*#__PURE__*/React__default.createElement("video", {
      className: classNames('cueplayer-react-video', this.props.className),
      id: videoId,
      crossOrigin: crossOrigin,
      ref: c => {
        this.video = c;
      },
      muted: muted,
      preload: preload,
      loop: loop,
      playsInline: playsInline,
      autoPlay: autoPlay,
      poster: poster,
      src: src,
      onLoadStart: this.handleLoadStart,
      onWaiting: this.handleWaiting,
      onCanPlay: this.handleCanPlay,
      onCanPlayThrough: this.handleCanPlayThrough,
      onPlaying: this.handlePlaying,
      onEnded: this.handleEnded,
      onSeeking: this.handleSeeking,
      onSeeked: this.handleSeeked,
      onPlay: this.handlePlay,
      onPause: this.handlePause,
      onProgress: this.handleProgress,
      onDurationChange: this.handleDurationChange,
      onError: this.handleError,
      onSuspend: this.handleSuspend,
      onAbort: this.handleAbort,
      onEmptied: this.handleEmptied,
      onStalled: this.handleStalled,
      onLoadedMetadata: this.handleLoadedMetaData,
      onLoadedData: this.handleLoadedData,
      onTimeUpdate: this.handleTimeUpdate,
      onRateChange: this.handleRateChange,
      onVolumeChange: this.handleVolumeChange,
      tabIndex: "-1"
    }, this.renderChildren());
  }

}
Video.propTypes = propTypes$3;
Video.displayName = 'Video';

const propTypes$4 = {
  manager: PropTypes.object,
  className: PropTypes.string
};
class Bezel extends Component {
  constructor(props) {
    super(props);
    this.timer = null;
    props.manager.subscribeToOperationStateChange(this.handleStateChange.bind(this));
    this.state = {
      hidden: true,
      operation: {}
    };
  }

  handleStateChange(state, prevState) {
    if (state.count !== prevState.count && state.operation.source === 'shortcut') {
      if (this.timer) {
        // previous animation is not finished
        clearTimeout(this.timer); // cancel it

        this.timer = null;
      } // show it
      // update operation


      this.setState({
        hidden: false,
        count: state.count,
        operation: state.operation
      }); // hide it after 0.5s

      this.timer = setTimeout(() => {
        this.setState({
          hidden: true
        });
        this.timer = null;
      }, 500);
    }
  }

  render() {
    // only displays for shortcut so far
    if (this.state.operation.source !== 'shortcut') {
      return null;
    }

    const style = this.state.hidden ? {
      display: 'none'
    } : null;
    return /*#__PURE__*/React__default.createElement("div", {
      className: classNames({
        'cueplayer-react-bezel': true,
        'cueplayer-react-bezel-animation': this.state.count % 2 === 0,
        'cueplayer-react-bezel-animation-alt': this.state.count % 2 === 1
      }, this.props.className),
      style: style,
      role: "status",
      "aria-label": this.state.operation.action
    }, /*#__PURE__*/React__default.createElement("div", {
      className: classNames('cueplayer-react-bezel-icon', "cueplayer-react-bezel-icon-" + this.state.operation.action)
    }));
  }

}
Bezel.propTypes = propTypes$4;
Bezel.displayName = 'Bezel';

/**
 * Offset Left
 * getBoundingClientRect technique from
 * John Resig http://ejohn.org/blog/getboundingclientrect-is-awesome/
 *
 * @function findElPosition
 * @param {ReactNodeRef} el React Node ref from which to get offset
 * @return {Object}
 */
function findElPosition(el) {
  let box;

  if (el.getBoundingClientRect && el.parentNode) {
    box = el.getBoundingClientRect();
  }

  if (!box) {
    return {
      left: 0,
      top: 0
    };
  }

  const {
    body,
    documentElement: docEl
  } = document;
  const clientLeft = docEl.clientLeft || body.clientLeft || 0;
  const scrollLeft = window.pageXOffset || body.scrollLeft;
  const left = box.left + scrollLeft - clientLeft;
  const clientTop = docEl.clientTop || body.clientTop || 0;
  const scrollTop = window.pageYOffset || body.scrollTop;
  const top = box.top + scrollTop - clientTop; // Android sometimes returns slightly off decimal values, so need to round

  return {
    left: Math.round(left),
    top: Math.round(top)
  };
}
/**
 * Get pointer position in a React Node ref
 * Returns an object with x and y coordinates.
 * The base on the coordinates are the bottom left of the element.
 *
 * @function getPointerPosition
 * @param {ReactNodeRef} el React Node ref on which to get the pointer position on
 * @param {Event} event Event object
 * @return {Object} This object will have x and y coordinates corresponding to the mouse position
 */

function getPointerPosition(el, event) {
  const position = {};
  const box = findElPosition(el);
  const boxW = el.offsetWidth;
  const boxH = el.offsetHeight;
  const boxY = box.top;
  const boxX = box.left;
  let evtPageY = event.pageY;
  let evtPageX = event.pageX;

  if (event.changedTouches) {
    evtPageX = event.changedTouches[0].pageX;
    evtPageY = event.changedTouches[0].pageY;
  }

  position.y = Math.max(0, Math.min(1, (boxY - evtPageY + boxH) / boxH));
  position.x = Math.max(0, Math.min(1, (evtPageX - boxX) / boxW));
  return position;
} // blur an element

function focusNode(reactNode) {
  if (reactNode && reactNode.focus) {
    reactNode.focus();
  }
} // check if an element has a class name

function hasClass(elm, cls) {
  const classes = elm.className.split(' ');

  for (let i = 0; i < classes.length; i++) {
    if (classes[i].toLowerCase() === cls.toLowerCase()) {
      return true;
    }
  }

  return false;
}

const propTypes$5 = {
  clickable: PropTypes.bool,
  dblclickable: PropTypes.bool,
  manager: PropTypes.object,
  actions: PropTypes.object,
  player: PropTypes.object,
  shortcuts: PropTypes.array
};
const defaultProps$1 = {
  clickable: true,
  dblclickable: true
};
class Shortcut extends Component {
  constructor(props) {
    super(props);
    this.defaultShortcuts = [{
      keyCode: 32,
      // spacebar
      handle: this.togglePlay
    }, {
      keyCode: 75,
      // k
      handle: this.togglePlay
    }, {
      keyCode: 70,
      // f
      handle: this.toggleFullscreen
    }, {
      keyCode: 37,
      // Left arrow
      handle: (player, actions) => {
        if (!player.hasStarted) {
          return;
        }

        actions.replay(5, {
          action: 'replay-5',
          source: 'shortcut'
        }); // Go back 5 seconds
      }
    }, {
      keyCode: 74,
      // j
      handle: (player, actions) => {
        if (!player.hasStarted) {
          return;
        }

        actions.replay(10, {
          action: 'replay-10',
          source: 'shortcut'
        }); // Go back 10 seconds
      }
    }, {
      keyCode: 39,
      // Right arrow
      handle: (player, actions) => {
        if (!player.hasStarted) {
          return;
        }

        actions.forward(5, {
          action: 'forward-5',
          source: 'shortcut'
        }); // Go forward 5 seconds
      }
    }, {
      keyCode: 76,
      // l
      handle: (player, actions) => {
        if (!player.hasStarted) {
          return;
        }

        actions.forward(10, {
          action: 'forward-10',
          source: 'shortcut'
        }); // Go forward 10 seconds
      }
    }, {
      keyCode: 36,
      // Home
      handle: (player, actions) => {
        if (!player.hasStarted) {
          return;
        }

        actions.seek(0); // Go to beginning of video
      }
    }, {
      keyCode: 35,
      // End
      handle: (player, actions) => {
        if (!player.hasStarted) {
          return;
        } // Go to end of video


        actions.seek(player.duration);
      }
    }, {
      keyCode: 38,
      // Up arrow
      handle: (player, actions) => {
        // Increase volume 5%
        let v = player.volume + 0.05;

        if (v > 1) {
          v = 1;
        }

        actions.changeVolume(v, {
          action: 'volume-up',
          source: 'shortcut'
        });
      }
    }, {
      keyCode: 40,
      // Down arrow
      handle: (player, actions) => {
        // Decrease volume 5%
        let v = player.volume - 0.05;

        if (v < 0) {
          v = 0;
        }

        const action = v > 0 ? 'volume-down' : 'volume-off';
        actions.changeVolume(v, {
          action,
          source: 'shortcut'
        });
      }
    }, {
      keyCode: 190,
      // Shift + >
      shift: true,
      handle: (player, actions) => {
        // Increase speed
        let {
          playbackRate
        } = player;

        if (playbackRate >= 1.5) {
          playbackRate = 2;
        } else if (playbackRate >= 1.25) {
          playbackRate = 1.5;
        } else if (playbackRate >= 1.0) {
          playbackRate = 1.25;
        } else if (playbackRate >= 0.5) {
          playbackRate = 1.0;
        } else if (playbackRate >= 0.25) {
          playbackRate = 0.5;
        } else if (playbackRate >= 0) {
          playbackRate = 0.25;
        }

        actions.changeRate(playbackRate, {
          action: 'fast-forward',
          source: 'shortcut'
        });
      }
    }, {
      keyCode: 188,
      // Shift + <
      shift: true,
      handle: (player, actions) => {
        // Decrease speed
        let {
          playbackRate
        } = player;

        if (playbackRate <= 0.5) {
          playbackRate = 0.25;
        } else if (playbackRate <= 1.0) {
          playbackRate = 0.5;
        } else if (playbackRate <= 1.25) {
          playbackRate = 1.0;
        } else if (playbackRate <= 1.5) {
          playbackRate = 1.25;
        } else if (playbackRate <= 2) {
          playbackRate = 1.5;
        }

        actions.changeRate(playbackRate, {
          action: 'fast-rewind',
          source: 'shortcut'
        });
      }
    }];
    this.shortcuts = [...this.defaultShortcuts];
    this.mergeShortcuts = this.mergeShortcuts.bind(this);
    this.handleKeyPress = this.handleKeyPress.bind(this);
    this.handleClick = this.handleClick.bind(this);
    this.handleDoubleClick = this.handleDoubleClick.bind(this);
  }

  componentDidMount() {
    this.mergeShortcuts();
    document.addEventListener('keydown', this.handleKeyPress);
    document.addEventListener('click', this.handleClick);
    document.addEventListener('dblclick', this.handleDoubleClick);
  }

  componentDidUpdate(prevProps) {
    if (prevProps.shortcuts !== this.props.shortcuts) {
      this.mergeShortcuts();
    }
  }

  componentWillUnmount() {
    document.removeEventListener('keydown', this.handleKeyPress);
    document.removeEventListener('click', this.handleClick);
    document.removeEventListener('dblclick', this.handleDoubleClick);
  } // merge the shortcuts from props


  mergeShortcuts() {
    const getShortcutKey = (_ref) => {
      let {
        keyCode = 0,
        ctrl = false,
        shift = false,
        alt = false
      } = _ref;
      return keyCode + ":" + ctrl + ":" + shift + ":" + alt;
    };

    const defaultShortcuts = this.defaultShortcuts.reduce((shortcuts, shortcut) => Object.assign(shortcuts, {
      [getShortcutKey(shortcut)]: shortcut
    }), {});
    const mergedShortcuts = (this.props.shortcuts || []).reduce((shortcuts, shortcut) => {
      const {
        keyCode,
        handle
      } = shortcut;

      if (keyCode && typeof handle === 'function') {
        return Object.assign(shortcuts, {
          [getShortcutKey(shortcut)]: shortcut
        });
      }

      return shortcuts;
    }, defaultShortcuts);

    const gradeShortcut = s => {
      let score = 0;
      const ps = ['ctrl', 'shift', 'alt'];
      ps.forEach(key => {
        if (s[key]) {
          score++;
        }
      });
      return score;
    };

    this.shortcuts = Object.keys(mergedShortcuts).map(key => mergedShortcuts[key]).sort((a, b) => gradeShortcut(b) - gradeShortcut(a));
  }

  togglePlay(player, actions) {
    if (player.paused) {
      actions.play({
        action: 'play',
        source: 'shortcut'
      });
    } else {
      actions.pause({
        action: 'pause',
        source: 'shortcut'
      });
    }
  }

  toggleFullscreen(player, actions) {
    actions.toggleFullscreen(player);
  }

  handleKeyPress(e) {
    const {
      player,
      actions
    } = this.props;

    if (!player.isActive) {
      return;
    }

    if (document.activeElement && (hasClass(document.activeElement, 'cueplayer-react-control') || hasClass(document.activeElement, 'cueplayer-react-menu-button-active') || // || hasClass(document.activeElement, 'cueplayer-react-slider')
    hasClass(document.activeElement, 'cueplayer-react-big-play-button'))) {
      return;
    }

    const keyCode = e.keyCode || e.which;
    const ctrl = e.ctrlKey || e.metaKey;
    const shift = e.shiftKey;
    const alt = e.altKey;
    const shortcut = this.shortcuts.filter(s => {
      if (!s.keyCode || s.keyCode - keyCode !== 0) {
        return false;
      }

      if (s.ctrl !== undefined && s.ctrl !== ctrl || s.shift !== undefined && s.shift !== shift || s.alt !== undefined && s.alt !== alt) {
        return false;
      }

      return true;
    })[0];

    if (shortcut) {
      shortcut.handle(player, actions);
      e.preventDefault();
    }
  } // only if player is active and player is ready


  canBeClicked(player, e) {
    if (!player.isActive || e.target.nodeName !== 'VIDEO' || player.readyState !== 4) {
      return false;
    }

    return true;
  }

  handleClick(e) {
    const {
      player,
      actions,
      clickable
    } = this.props;

    if (!this.canBeClicked(player, e) || !clickable) {
      return;
    }

    this.togglePlay(player, actions); // e.preventDefault();
  }

  handleDoubleClick(e) {
    const {
      player,
      actions,
      dblclickable
    } = this.props;

    if (!this.canBeClicked(player, e) || !dblclickable) {
      return;
    }

    this.toggleFullscreen(player, actions); // e.preventDefault();
  } // this component dose not render anything
  // it's just for the key down event


  render() {
    return null;
  }

}
Shortcut.propTypes = propTypes$5;
Shortcut.defaultProps = defaultProps$1;
Shortcut.displayName = 'Shortcut';

const propTypes$6 = {
  className: PropTypes.string,
  onMouseDown: PropTypes.func,
  onMouseMove: PropTypes.func,
  stepForward: PropTypes.func,
  stepBack: PropTypes.func,
  sliderActive: PropTypes.func,
  sliderInactive: PropTypes.func,
  onMouseUp: PropTypes.func,
  onFocus: PropTypes.func,
  onBlur: PropTypes.func,
  onClick: PropTypes.func,
  getPercent: PropTypes.func,
  vertical: PropTypes.bool,
  children: PropTypes.node,
  label: PropTypes.string,
  valuenow: PropTypes.string,
  valuetext: PropTypes.string
};
class Slider extends Component {
  constructor(props) {
    super(props);
    this.handleMouseDown = this.handleMouseDown.bind(this);
    this.handleMouseMove = this.handleMouseMove.bind(this);
    this.handleMouseUp = this.handleMouseUp.bind(this);
    this.handleFocus = this.handleFocus.bind(this);
    this.handleBlur = this.handleBlur.bind(this);
    this.handleClick = this.handleClick.bind(this);
    this.handleKeyPress = this.handleKeyPress.bind(this);
    this.stepForward = this.stepForward.bind(this);
    this.stepBack = this.stepBack.bind(this);
    this.calculateDistance = this.calculateDistance.bind(this);
    this.getProgress = this.getProgress.bind(this);
    this.renderChildren = this.renderChildren.bind(this);
    this.state = {
      active: false
    };
  }

  componentWillUnmount() {
    document.removeEventListener('mousemove', this.handleMouseMove, true);
    document.removeEventListener('mouseup', this.handleMouseUp, true);
    document.removeEventListener('touchmove', this.handleMouseMove, true);
    document.removeEventListener('touchend', this.handleMouseUp, true);
    document.removeEventListener('keydown', this.handleKeyPress, true);
  }

  getProgress() {
    const {
      getPercent
    } = this.props;

    if (!getPercent) {
      return 0;
    }

    let progress = getPercent(); // Protect against no duration and other division issues

    if (typeof progress !== 'number' || progress < 0 || progress === Infinity) {
      progress = 0;
    }

    return progress;
  }

  handleMouseDown(event) {
    const {
      onMouseDown
    } = this.props; // event.preventDefault();
    // event.stopPropagation();

    document.addEventListener('mousemove', this.handleMouseMove, true);
    document.addEventListener('mouseup', this.handleMouseUp, true);
    document.addEventListener('touchmove', this.handleMouseMove, true);
    document.addEventListener('touchend', this.handleMouseUp, true);
    this.setState({
      active: true
    });

    if (this.props.sliderActive) {
      this.props.sliderActive(event);
    }

    this.handleMouseMove(event);

    if (onMouseDown) {
      onMouseDown(event);
    }
  }

  handleMouseMove(event) {
    const {
      onMouseMove
    } = this.props;

    if (onMouseMove) {
      onMouseMove(event);
    }
  }

  handleMouseUp(event) {
    // On iOS safari, a subsequent mouseup event will be fired after touchend.
    // Its weird event positions make the player seek a wrong time.
    // calling preventDefault (at touchend phase) will prevent the mouseup event
    event.preventDefault();
    const {
      onMouseUp
    } = this.props;
    document.removeEventListener('mousemove', this.handleMouseMove, true);
    document.removeEventListener('mouseup', this.handleMouseUp, true);
    document.removeEventListener('touchmove', this.handleMouseMove, true);
    document.removeEventListener('touchend', this.handleMouseUp, true);
    this.setState({
      active: false
    });

    if (this.props.sliderInactive) {
      this.props.sliderInactive(event);
    }

    if (onMouseUp) {
      onMouseUp(event);
    }
  }

  handleFocus(e) {
    document.addEventListener('keydown', this.handleKeyPress, true);

    if (this.props.onFocus) {
      this.props.onFocus(e);
    }
  }

  handleBlur(e) {
    document.removeEventListener('keydown', this.handleKeyPress, true);

    if (this.props.onBlur) {
      this.props.onBlur(e);
    }
  }

  handleClick(event) {
    event.preventDefault(); // event.stopPropagation();

    if (this.props.onClick) {
      this.props.onClick(event);
    }
  }

  handleKeyPress(event) {
    if (event.which === 37 || event.which === 40) {
      // Left and Down Arrows
      event.preventDefault();
      event.stopPropagation();
      this.stepBack();
    } else if (event.which === 38 || event.which === 39) {
      // Up and Right Arrows
      event.preventDefault();
      event.stopPropagation();
      this.stepForward();
    }
  }

  stepForward() {
    if (this.props.stepForward) {
      this.props.stepForward();
    }
  }

  stepBack() {
    if (this.props.stepBack) {
      this.props.stepBack();
    }
  }

  calculateDistance(event) {
    const node = this.slider;
    const position = getPointerPosition(node, event);

    if (this.props.vertical) {
      return position.y;
    }

    return position.x;
  }

  renderChildren() {
    const progress = this.getProgress();
    const percentage = (progress * 100).toFixed(2) + "%";
    return React__default.Children.map(this.props.children, child => /*#__PURE__*/React__default.cloneElement(child, {
      progress,
      percentage
    }));
  }

  render() {
    const {
      vertical,
      label,
      valuenow,
      valuetext
    } = this.props;
    return /*#__PURE__*/React__default.createElement("div", {
      className: classNames(this.props.className, {
        'cueplayer-react-slider-vertical': vertical,
        'cueplayer-react-slider-horizontal': !vertical,
        'cueplayer-react-sliding': this.state.active
      }, 'cueplayer-react-slider'),
      ref: c => {
        this.slider = c;
      },
      tabIndex: "0",
      role: "slider",
      onMouseDown: this.handleMouseDown,
      onTouchStart: this.handleMouseDown,
      onFocus: this.handleFocus,
      onBlur: this.handleBlur,
      onClick: this.handleClick,
      "aria-label": label || '',
      "aria-valuenow": valuenow || '',
      "aria-valuetext": valuetext || '',
      "aria-valuemin": 0,
      "aria-valuemax": 100
    }, this.renderChildren());
  }

}
Slider.propTypes = propTypes$6;
Slider.displayName = 'Slider';

const propTypes$7 = {
  currentTime: PropTypes.number,
  duration: PropTypes.number,
  percentage: PropTypes.string,
  className: PropTypes.string
}; // Shows play progress

function PlayProgressBar(_ref) {
  let {
    currentTime,
    duration,
    percentage,
    className
  } = _ref;
  return /*#__PURE__*/React__default.createElement("div", {
    "data-current-time": formatTime(currentTime, duration),
    className: classNames('cueplayer-react-play-progress cueplayer-react-slider-bar', className),
    style: {
      width: percentage
    }
  }, /*#__PURE__*/React__default.createElement("span", {
    className: "cueplayer-react-control-text"
  }, "Progress: " + percentage));
}
PlayProgressBar.propTypes = propTypes$7;
PlayProgressBar.displayName = 'PlayProgressBar';

const propTypes$8 = {
  duration: PropTypes.number,
  buffered: PropTypes.object,
  className: PropTypes.string
}; // Shows load progress

function LoadProgressBar(_ref) {
  let {
    buffered,
    duration,
    className
  } = _ref;

  if (!buffered || !buffered.length) {
    return null;
  }

  let bufferedEnd = buffered.end(buffered.length - 1);
  const style = {};

  if (bufferedEnd > duration) {
    bufferedEnd = duration;
  } // get the percent width of a time compared to the total end


  function percentify(time, end) {
    const percent = time / end || 0; // no NaN

    return (percent >= 1 ? 1 : percent) * 100 + "%";
  } // the width of the progress bar


  style.width = percentify(bufferedEnd, duration);
  let parts = []; // add child elements to represent the individual buffered time ranges

  for (let i = 0; i < buffered.length; i++) {
    const start = buffered.start(i);
    const end = buffered.end(i); // set the percent based on the width of the progress bar (bufferedEnd)

    const part = /*#__PURE__*/React__default.createElement("div", {
      style: {
        left: percentify(start, bufferedEnd),
        width: percentify(end - start, bufferedEnd)
      },
      key: "part-" + i
    });
    parts.push(part);
  }

  if (parts.length === 0) {
    parts = null;
  }

  return /*#__PURE__*/React__default.createElement("div", {
    style: style,
    className: classNames('cueplayer-react-load-progress', className)
  }, /*#__PURE__*/React__default.createElement("span", {
    className: "cueplayer-react-control-text"
  }, "Loaded: 0%"), parts);
}
LoadProgressBar.propTypes = propTypes$8;
LoadProgressBar.displayName = 'LoadProgressBar';

function MouseTimeDisplay(_ref) {
  let {
    duration,
    mouseTime,
    className,
    text
  } = _ref;

  if (!mouseTime.time) {
    return null;
  }

  const time = text || formatTime(mouseTime.time, duration);
  return /*#__PURE__*/React__default.createElement("div", {
    className: classNames('cueplayer-react-mouse-display', className),
    style: {
      left: mouseTime.position + "px"
    },
    "data-current-time": time
  });
}

MouseTimeDisplay.propTypes = {
  duration: PropTypes.number,
  mouseTime: PropTypes.object,
  className: PropTypes.string
};
MouseTimeDisplay.displayName = 'MouseTimeDisplay';

const propTypes$9 = {
  player: PropTypes.object,
  mouseTime: PropTypes.object,
  actions: PropTypes.object,
  className: PropTypes.string
};
class SeekBar extends Component {
  constructor(props) {
    super(props);
    this.getPercent = this.getPercent.bind(this);
    this.getNewTime = this.getNewTime.bind(this);
    this.stepForward = this.stepForward.bind(this);
    this.stepBack = this.stepBack.bind(this);
    this.handleMouseDown = this.handleMouseDown.bind(this);
    this.handleMouseMove = this.handleMouseMove.bind(this);
    this.handleMouseUp = this.handleMouseUp.bind(this);
  }

  componentDidMount() {}

  componentDidUpdate() {}
  /**
   * Get percentage of video played
   *
   * @return {Number} Percentage played
   * @method getPercent
   */


  getPercent() {
    const {
      currentTime,
      seekingTime,
      duration
    } = this.props.player;
    const time = seekingTime || currentTime;
    const percent = time / duration;
    return percent >= 1 ? 1 : percent;
  }

  getNewTime(event) {
    const {
      player: {
        duration
      }
    } = this.props;
    const distance = this.slider.calculateDistance(event);
    const newTime = distance * duration; // Don't let video end while scrubbing.

    return newTime === duration ? newTime - 0.1 : newTime;
  }

  handleMouseDown() {}

  handleMouseUp(event) {
    const {
      actions
    } = this.props;
    const newTime = this.getNewTime(event); // Set new time (tell video to seek to new time)

    actions.seek(newTime);
    actions.handleEndSeeking(newTime);
  }

  handleMouseMove(event) {
    const {
      actions
    } = this.props;
    const newTime = this.getNewTime(event);
    actions.handleSeekingTime(newTime);
  }

  stepForward() {
    const {
      actions
    } = this.props;
    actions.forward(5);
  }

  stepBack() {
    const {
      actions
    } = this.props;
    actions.replay(5);
  }

  render() {
    const {
      player: {
        currentTime,
        seekingTime,
        duration,
        buffered
      },
      mouseTime
    } = this.props;
    const time = seekingTime || currentTime;
    return /*#__PURE__*/React__default.createElement(Slider, {
      ref: input => {
        this.slider = input;
      },
      label: "video progress bar",
      className: classNames('cueplayer-react-progress-holder', this.props.className),
      valuenow: (this.getPercent() * 100).toFixed(2),
      valuetext: formatTime(time, duration),
      onMouseDown: this.handleMouseDown,
      onMouseMove: this.handleMouseMove,
      onMouseUp: this.handleMouseUp,
      getPercent: this.getPercent,
      stepForward: this.stepForward,
      stepBack: this.stepBack
    }, /*#__PURE__*/React__default.createElement(LoadProgressBar, {
      buffered: buffered,
      currentTime: time,
      duration: duration
    }), /*#__PURE__*/React__default.createElement(MouseTimeDisplay, {
      duration: duration,
      mouseTime: mouseTime
    }), /*#__PURE__*/React__default.createElement(PlayProgressBar, {
      currentTime: time,
      duration: duration
    }));
  }

}
SeekBar.propTypes = propTypes$9;
SeekBar.displayName = 'SeekBar';

const propTypes$a = {
  player: PropTypes.object,
  className: PropTypes.string
};
class ProgressControl extends Component {
  constructor(props) {
    super(props);
    this.state = {
      mouseTime: {
        time: null,
        position: 0
      }
    };
    this.handleMouseMoveThrottle = this.handleMouseMove.bind(this);
  }

  handleMouseMove(event) {
    if (!event.pageX) {
      return;
    }

    const {
      player: {
        duration
      }
    } = this.props;
    const node = this.seekBar;
    const newTime = getPointerPosition(node, event).x * duration;
    const position = event.pageX - findElPosition(node).left;
    this.setState({
      mouseTime: {
        time: newTime,
        position
      }
    });
  }

  render() {
    const {
      className
    } = this.props;
    return /*#__PURE__*/React__default.createElement("div", {
      onMouseMove: this.handleMouseMoveThrottle,
      className: classNames('cueplayer-react-progress-control cueplayer-react-control', className)
    }, /*#__PURE__*/React__default.createElement(SeekBar, _extends({
      mouseTime: this.state.mouseTime,
      ref: c => {
        this.seekBar = c;
      }
    }, this.props)));
  }

}
ProgressControl.propTypes = propTypes$a;
ProgressControl.displayName = 'ProgressControl';

const propTypes$b = {
  actions: PropTypes.object,
  player: PropTypes.object,
  className: PropTypes.string
};
class PlayToggle extends Component {
  constructor(props) {
    super(props);
    this.handleClick = this.handleClick.bind(this);
  }

  handleClick() {
    const {
      actions,
      player
    } = this.props;

    if (player.paused) {
      actions.play();
    } else {
      actions.pause();
    }
  }

  render() {
    const {
      player,
      className
    } = this.props;
    const controlText = player.paused ? 'Play' : 'Pause';
    return /*#__PURE__*/React__default.createElement("button", {
      ref: c => {
        this.button = c;
      },
      className: classNames(className, {
        'cueplayer-react-play-control': true,
        'cueplayer-react-control': true,
        'cueplayer-react-button': true,
        'cueplayer-react-paused': player.paused,
        'cueplayer-react-playing': !player.paused
      }),
      type: "button",
      tabIndex: "0",
      onClick: this.handleClick
    }, /*#__PURE__*/React__default.createElement("span", {
      className: "cueplayer-react-control-text"
    }, controlText));
  }

}
PlayToggle.propTypes = propTypes$b;
PlayToggle.displayName = 'PlayToggle';

const propTypes$c = {
  actions: PropTypes.object,
  className: PropTypes.string,
  seconds: PropTypes.oneOf([5, 10, 30])
};
const defaultProps$2 = {
  seconds: 10
};
var ForwardReplayControl = (mode => {
  class ForwardReplayControl extends Component {
    constructor(props) {
      super(props);
      this.handleClick = this.handleClick.bind(this);
    }

    handleClick() {
      const {
        actions,
        seconds
      } = this.props; // Depends mode to implement different actions

      if (mode === 'forward') {
        actions.forward(seconds);
      } else {
        actions.replay(seconds);
      }
    }

    render() {
      const {
        seconds,
        className
      } = this.props;
      const classNames = ['cueplayer-react-control', 'cueplayer-react-button', 'cueplayer-react-icon'];
      classNames.push("cueplayer-react-icon-" + mode + "-" + seconds, "cueplayer-react-" + mode + "-control");

      if (className) {
        classNames.push(className);
      }

      return /*#__PURE__*/React__default.createElement("button", {
        ref: c => {
          this.button = c;
        },
        className: classNames.join(' '),
        type: "button",
        onClick: this.handleClick
      }, /*#__PURE__*/React__default.createElement("span", {
        className: "cueplayer-react-control-text"
      }, mode + " " + seconds + " seconds"));
    }

  }

  ForwardReplayControl.propTypes = propTypes$c;
  ForwardReplayControl.defaultProps = defaultProps$2;
  return ForwardReplayControl;
});

const ForwardControl = ForwardReplayControl('forward');
ForwardControl.displayName = 'ForwardControl';

const ReplayControl = ForwardReplayControl('replay');
ReplayControl.displayName = 'ReplayControl';

const propTypes$d = {
  actions: PropTypes.object,
  player: PropTypes.object,
  className: PropTypes.string,
  fullscreenElement: PropTypes.any
};
class FullscreenToggle extends Component {
  constructor(props) {
    super(props);
    this.handleClick = this.handleClick.bind(this);
    this.handleFullScreenChange = this.handleFullScreenChange.bind(this);
  }

  componentDidMount() {
    fullscreen.addEventListener(this.handleFullScreenChange);
  }

  componentWillUnmount() {
    fullscreen.removeEventListener(this.handleFullScreenChange);
  }

  handleFullScreenChange(event) {
    const {
      fullscreenElement,
      actions
    } = this.props;

    if (event.target === fullscreenElement) {
      actions.handleFullscreenChange(fullscreen.isFullscreen);
    }
  }

  handleClick() {
    const {
      player,
      actions,
      fullscreenElement
    } = this.props;
    actions.toggleFullscreen(player, fullscreenElement);
  }

  render() {
    const {
      player,
      className
    } = this.props;
    return /*#__PURE__*/React__default.createElement("button", {
      className: classNames(className, {
        'cueplayer-react-icon-fullscreen-exit': player.isFullscreen,
        'cueplayer-react-icon-fullscreen': !player.isFullscreen
      }, 'cueplayer-react-fullscreen-control cueplayer-react-control cueplayer-react-button cueplayer-react-icon'),
      ref: c => {
        this.button = c;
      },
      type: "button",
      tabIndex: "0",
      onClick: this.handleClick
    }, /*#__PURE__*/React__default.createElement("span", {
      className: "cueplayer-react-control-text"
    }, "Non-Fullscreen"));
  }

}
FullscreenToggle.propTypes = propTypes$d;
FullscreenToggle.displayName = 'FullscreenToggle';

const propTypes$e = {
  player: PropTypes.object,
  className: PropTypes.string
};

function RemainingTimeDisplay(_ref) {
  let {
    player: {
      currentTime,
      duration
    },
    className
  } = _ref;
  const remainingTime = duration - currentTime;
  const formattedTime = formatTime(remainingTime);
  return /*#__PURE__*/React__default.createElement("div", {
    className: classNames('cueplayer-react-remaining-time cueplayer-react-time-control cueplayer-react-control', className)
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "cueplayer-react-remaining-time-display",
    "aria-live": "off"
  }, /*#__PURE__*/React__default.createElement("span", {
    className: "cueplayer-react-control-text"
  }, "Remaining Time "), "-" + formattedTime));
}

RemainingTimeDisplay.propTypes = propTypes$e;
RemainingTimeDisplay.displayName = 'RemainingTimeDisplay';

const propTypes$f = {
  player: PropTypes.object,
  className: PropTypes.string
};

function CurrentTimeDisplay(_ref) {
  let {
    player: {
      currentTime,
      duration
    },
    className
  } = _ref;
  const formattedTime = formatTime(currentTime, duration);
  return /*#__PURE__*/React__default.createElement("div", {
    className: classNames('cueplayer-react-current-time cueplayer-react-time-control cueplayer-react-control', className)
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "cueplayer-react-current-time-display",
    "aria-live": "off"
  }, /*#__PURE__*/React__default.createElement("span", {
    className: "cueplayer-react-control-text"
  }, "Current Time "), formattedTime));
}

CurrentTimeDisplay.propTypes = propTypes$f;
CurrentTimeDisplay.displayName = 'CurrentTimeDisplay';

const propTypes$g = {
  player: PropTypes.object,
  className: PropTypes.string
};

function DurationDisplay(_ref) {
  let {
    player: {
      duration
    },
    className
  } = _ref;
  const formattedTime = formatTime(duration);
  return /*#__PURE__*/React__default.createElement("div", {
    className: classNames(className, 'cueplayer-react-duration cueplayer-react-time-control cueplayer-react-control')
  }, /*#__PURE__*/React__default.createElement("div", {
    className: "cueplayer-react-duration-display",
    "aria-live": "off"
  }, /*#__PURE__*/React__default.createElement("span", {
    className: "cueplayer-react-control-text"
  }, "Duration Time "), formattedTime));
}

DurationDisplay.propTypes = propTypes$g;
DurationDisplay.displayName = 'DurationDisplay';

const propTypes$h = {
  separator: PropTypes.string,
  className: PropTypes.string
};
function TimeDivider(_ref) {
  let {
    separator,
    className
  } = _ref;
  const separatorText = separator || '/';
  return /*#__PURE__*/React__default.createElement("div", {
    className: classNames('cueplayer-react-time-control cueplayer-react-time-divider', className),
    dir: "ltr"
  }, /*#__PURE__*/React__default.createElement("div", null, /*#__PURE__*/React__default.createElement("span", null, separatorText)));
}
TimeDivider.propTypes = propTypes$h;
TimeDivider.displayName = 'TimeDivider';

const propTypes$i = {
  tagName: PropTypes.string,
  onClick: PropTypes.func.isRequired,
  onFocus: PropTypes.func,
  onBlur: PropTypes.func,
  className: PropTypes.string
};
const defaultProps$3 = {
  tagName: 'div'
};
class ClickableComponent extends Component {
  constructor(props) {
    super(props);
    this.handleClick = this.handleClick.bind(this);
    this.handleFocus = this.handleFocus.bind(this);
    this.handleBlur = this.handleBlur.bind(this);
    this.handleKeypress = this.handleKeypress.bind(this);
  }

  componentWillUnmount(e) {
    this.handleBlur(e);
  }

  handleKeypress(event) {
    // Support Space (32) or Enter (13) key operation to fire a click event
    if (event.which === 32 || event.which === 13) {
      event.preventDefault();
      this.handleClick(event);
    }
  }

  handleClick(event) {
    const {
      onClick
    } = this.props;
    onClick(event);
  }

  handleFocus(e) {
    document.addEventListener('keydown', this.handleKeypress);

    if (this.props.onFocus) {
      this.props.onFocus(e);
    }
  }

  handleBlur(e) {
    document.removeEventListener('keydown', this.handleKeypress);

    if (this.props.onBlur) {
      this.props.onBlur(e);
    }
  }

  render() {
    const Tag = this.props.tagName;

    const props = _extends({}, this.props);

    delete props.tagName;
    delete props.className;
    return /*#__PURE__*/React__default.createElement(Tag, _extends({
      className: classNames(this.props.className),
      role: "button",
      tabIndex: "0",
      onClick: this.handleClick,
      onFocus: this.handleFocus,
      onBlur: this.handleBlur
    }, props));
  }

}
ClickableComponent.propTypes = propTypes$i;
ClickableComponent.defaultProps = defaultProps$3;
ClickableComponent.displayName = 'ClickableComponent';

const propTypes$j = {
  player: PropTypes.object,
  children: PropTypes.any
};
class Popup extends Component {
  constructor(props) {
    super(props);
    this.handleClick = this.handleClick.bind(this);
  }

  handleClick(event) {
    event.preventDefault(); // event.stopPropagation();
  }

  render() {
    const {
      children
    } = this.props;
    return /*#__PURE__*/React__default.createElement("div", {
      className: "cueplayer-react-menu",
      onClick: this.handleClick
    }, /*#__PURE__*/React__default.createElement("div", {
      className: "cueplayer-react-menu-content"
    }, children));
  }

}
Popup.propTypes = propTypes$j;
Popup.displayName = 'Popup';

const propTypes$k = {
  inline: PropTypes.bool,
  onClick: PropTypes.func.isRequired,
  onFocus: PropTypes.func,
  onBlur: PropTypes.func,
  className: PropTypes.string
};
const defaultProps$4 = {
  inline: true
};
function PopupButton(props) {
  const {
    inline,
    className
  } = props;

  const ps = _extends({}, props);

  delete ps.children;
  delete ps.inline;
  delete ps.className;
  return /*#__PURE__*/React__default.createElement(ClickableComponent, _extends({
    className: classNames(className, {
      'cueplayer-react-menu-button-inline': !!inline,
      'cueplayer-react-menu-button-popup': !inline
    }, 'cueplayer-react-control cueplayer-react-button cueplayer-react-menu-button')
  }, ps), /*#__PURE__*/React__default.createElement(Popup, props));
}
PopupButton.propTypes = propTypes$k;
PopupButton.defaultProps = defaultProps$4;
PopupButton.displayName = 'PopupButton';

const propTypes$l = {
  percentage: PropTypes.string,
  vertical: PropTypes.bool,
  className: PropTypes.string
};
const defaultProps$5 = {
  percentage: '100%',
  vertical: false
};

function VolumeLevel(_ref) {
  let {
    percentage,
    vertical,
    className
  } = _ref;
  const style = {};

  if (vertical) {
    style.height = percentage;
  } else {
    style.width = percentage;
  }

  return /*#__PURE__*/React__default.createElement("div", {
    className: classNames(className, 'cueplayer-react-volume-level'),
    style: style
  }, /*#__PURE__*/React__default.createElement("span", {
    className: "cueplayer-react-control-text"
  }));
}

VolumeLevel.propTypes = propTypes$l;
VolumeLevel.defaultProps = defaultProps$5;
VolumeLevel.displayName = 'VolumeLevel';

const propTypes$m = {
  actions: PropTypes.object,
  player: PropTypes.object,
  className: PropTypes.string,
  onFocus: PropTypes.func,
  onBlur: PropTypes.func
};

class VolumeBar extends Component {
  constructor(props) {
    super(props);
    this.state = {
      percentage: '0%'
    };
    this.handleMouseMove = this.handleMouseMove.bind(this);
    this.handlePercentageChange = this.handlePercentageChange.bind(this);
    this.checkMuted = this.checkMuted.bind(this);
    this.getPercent = this.getPercent.bind(this);
    this.stepForward = this.stepForward.bind(this);
    this.stepBack = this.stepBack.bind(this);
    this.handleFocus = this.handleFocus.bind(this);
    this.handleBlur = this.handleBlur.bind(this);
    this.handleClick = this.handleClick.bind(this);
  }

  componentDidMount() {}

  getPercent() {
    const {
      player
    } = this.props;

    if (player.muted) {
      return 0;
    }

    return player.volume;
  }

  checkMuted() {
    const {
      player,
      actions
    } = this.props;

    if (player.muted) {
      actions.mute(false);
    }
  }

  handleMouseMove(event) {
    const {
      actions
    } = this.props;
    this.checkMuted();
    const distance = this.slider.calculateDistance(event);
    actions.changeVolume(distance);
  }

  stepForward() {
    const {
      player,
      actions
    } = this.props;
    this.checkMuted();
    actions.changeVolume(player.volume + 0.1);
  }

  stepBack() {
    const {
      player,
      actions
    } = this.props;
    this.checkMuted();
    actions.changeVolume(player.volume - 0.1);
  }

  handleFocus(e) {
    if (this.props.onFocus) {
      this.props.onFocus(e);
    }
  }

  handleBlur(e) {
    if (this.props.onBlur) {
      this.props.onBlur(e);
    }
  }

  handlePercentageChange(percentage) {
    if (percentage !== this.state.percentage) {
      this.setState({
        percentage
      });
    }
  }

  handleClick(event) {
    event.stopPropagation();
  }

  render() {
    const {
      player,
      className
    } = this.props;
    const volume = (player.volume * 100).toFixed(2);
    return /*#__PURE__*/React__default.createElement(Slider, _extends({
      ref: c => {
        this.slider = c;
      },
      label: "volume level",
      valuenow: volume,
      valuetext: volume + "%",
      onMouseMove: this.handleMouseMove,
      onFocus: this.handleFocus,
      onBlur: this.handleBlur,
      onClick: this.handleClick,
      sliderActive: this.handleFocus,
      sliderInactive: this.handleBlur,
      getPercent: this.getPercent,
      onPercentageChange: this.handlePercentageChange,
      stepForward: this.stepForward,
      stepBack: this.stepBack
    }, this.props, {
      className: classNames(className, 'cueplayer-react-volume-bar cueplayer-react-slider-bar')
    }), /*#__PURE__*/React__default.createElement(VolumeLevel, this.props));
  }

}

VolumeBar.propTypes = propTypes$m;
VolumeBar.displayName = 'VolumeBar';

const propTypes$n = {
  player: PropTypes.object,
  actions: PropTypes.object,
  vertical: PropTypes.bool,
  className: PropTypes.string,
  alwaysShowVolume: PropTypes.bool
};
const defaultProps$6 = {
  vertical: false
};

class VolumeMenuButton extends Component {
  constructor(props) {
    super(props);
    this.state = {
      active: false
    };
    this.handleClick = this.handleClick.bind(this);
    this.handleFocus = this.handleFocus.bind(this);
    this.handleBlur = this.handleBlur.bind(this);
  }

  get volumeLevel() {
    const {
      player: {
        volume,
        muted
      }
    } = this.props;
    let level = 3;

    if (volume === 0 || muted) {
      level = 0;
    } else if (volume < 0.33) {
      level = 1;
    } else if (volume < 0.67) {
      level = 2;
    }

    return level;
  }

  handleClick() {
    const {
      player,
      actions
    } = this.props;
    actions.mute(!player.muted);
  }

  handleFocus() {
    this.setState({
      active: true
    });
  }

  handleBlur() {
    this.setState({
      active: false
    });
  }

  render() {
    const {
      vertical,
      player,
      className
    } = this.props;
    const inline = !vertical;
    const level = this.volumeLevel;
    return /*#__PURE__*/React__default.createElement(PopupButton, {
      className: classNames(className, {
        'cueplayer-react-volume-menu-button-vertical': vertical,
        'cueplayer-react-volume-menu-button-horizontal': !vertical,
        'cueplayer-react-vol-muted': player.muted,
        'cueplayer-react-vol-0': level === 0 && !player.muted,
        'cueplayer-react-vol-1': level === 1,
        'cueplayer-react-vol-2': level === 2,
        'cueplayer-react-vol-3': level === 3,
        'cueplayer-react-slider-active': this.props.alwaysShowVolume || this.state.active,
        'cueplayer-react-lock-showing': this.props.alwaysShowVolume || this.state.active
      }, 'cueplayer-react-volume-menu-button'),
      onClick: this.handleClick,
      inline: inline
    }, /*#__PURE__*/React__default.createElement(VolumeBar, _extends({
      onFocus: this.handleFocus,
      onBlur: this.handleBlur
    }, this.props)));
  }

}

VolumeMenuButton.propTypes = propTypes$n;
VolumeMenuButton.defaultProps = defaultProps$6;
VolumeMenuButton.displayName = 'VolumeMenuButton';

const propTypes$o = {
  children: PropTypes.any
};
class Menu extends Component {
  constructor(props) {
    super(props);
    this.handleClick = this.handleClick.bind(this);
  }

  handleClick(event) {
    event.preventDefault(); // event.stopPropagation();
  }

  render() {
    return /*#__PURE__*/React__default.createElement("div", {
      className: "cueplayer-react-menu cueplayer-react-lock-showing",
      role: "presentation",
      onClick: this.handleClick
    }, /*#__PURE__*/React__default.createElement("ul", {
      className: "cueplayer-react-menu-content"
    }, this.props.children));
  }

}
Menu.propTypes = propTypes$o;
Menu.displayName = 'Menu';

const propTypes$p = {
  item: PropTypes.object,
  index: PropTypes.number,
  activateIndex: PropTypes.number,
  onSelectItem: PropTypes.func
};
class MenuItem extends Component {
  constructor(props) {
    super(props);
    this.handleClick = this.handleClick.bind(this);
  }

  handleClick() {
    const {
      index,
      onSelectItem
    } = this.props;
    onSelectItem(index);
  }

  render() {
    const {
      item,
      index,
      activateIndex
    } = this.props;
    return /*#__PURE__*/React__default.createElement("li", {
      className: classNames({
        'cueplayer-react-menu-item': true,
        'cueplayer-react-selected': index === activateIndex
      }),
      role: "menuitem",
      onClick: this.handleClick
    }, item.label, /*#__PURE__*/React__default.createElement("span", {
      className: "cueplayer-react-control-text"
    }));
  }

}
MenuItem.propTypes = propTypes$p;
MenuItem.displayName = 'MenuItem';

const propTypes$q = {
  inline: PropTypes.bool,
  items: PropTypes.array,
  className: PropTypes.string,
  onSelectItem: PropTypes.func,
  children: PropTypes.any,
  selectedIndex: PropTypes.number
};
class MenuButton extends Component {
  constructor(props) {
    super(props);
    this.state = {
      active: false,
      activateIndex: props.selectedIndex || 0
    };
    this.commitSelection = this.commitSelection.bind(this);
    this.activateMenuItem = this.activateMenuItem.bind(this);
    this.handleClick = this.handleClick.bind(this);
    this.renderMenu = this.renderMenu.bind(this);
    this.handleFocus = this.handleFocus.bind(this);
    this.handleBlur = this.handleBlur.bind(this);
    this.handleUpArrow = this.handleUpArrow.bind(this);
    this.handleDownArrow = this.handleDownArrow.bind(this);
    this.handleEscape = this.handleEscape.bind(this);
    this.handleReturn = this.handleReturn.bind(this);
    this.handleTab = this.handleTab.bind(this);
    this.handleKeyPress = this.handleKeyPress.bind(this);
    this.handleSelectItem = this.handleSelectItem.bind(this);
    this.handleIndexChange = this.handleIndexChange.bind(this);
  }

  componentDidUpdate(prevProps) {
    if (prevProps.selectedIndex !== this.props.selectedIndex) {
      this.activateMenuItem(this.props.selectedIndex);
    }
  }

  commitSelection(index) {
    this.setState({
      activateIndex: index
    });
    this.handleIndexChange(index);
  }

  activateMenuItem(index) {
    this.setState({
      activateIndex: index
    });
    this.handleIndexChange(index);
  }

  handleIndexChange(index) {
    const {
      onSelectItem
    } = this.props;
    onSelectItem(index);
  }

  handleClick() {
    this.setState(prevState => ({
      active: !prevState.active
    }));
  }

  handleFocus() {
    document.addEventListener('keydown', this.handleKeyPress);
  }

  handleBlur() {
    this.setState({
      active: false
    });
    document.removeEventListener('keydown', this.handleKeyPress);
  }

  handleUpArrow(e) {
    const {
      items
    } = this.props;

    if (this.state.active) {
      e.preventDefault();
      let newIndex = this.state.activateIndex - 1;

      if (newIndex < 0) {
        newIndex = items.length ? items.length - 1 : 0;
      }

      this.activateMenuItem(newIndex);
    }
  }

  handleDownArrow(e) {
    const {
      items
    } = this.props;

    if (this.state.active) {
      e.preventDefault();
      let newIndex = this.state.activateIndex + 1;

      if (newIndex >= items.length) {
        newIndex = 0;
      }

      this.activateMenuItem(newIndex);
    }
  }

  handleTab(e) {
    if (this.state.active) {
      e.preventDefault();
      this.commitSelection(this.state.activateIndex);
    }
  }

  handleReturn(e) {
    e.preventDefault();

    if (this.state.active) {
      this.commitSelection(this.state.activateIndex);
    } else {
      this.setState({
        active: true
      });
    }
  }

  handleEscape() {
    this.setState({
      active: false,
      activateIndex: 0
    });
  }

  handleKeyPress(event) {
    // Escape (27) key
    if (event.which === 27) {
      this.handleEscape(event);
    } else if (event.which === 9) {
      // Tab (9) key
      this.handleTab(event);
    } else if (event.which === 13) {
      // Enter (13) key
      this.handleReturn(event);
    } else if (event.which === 38) {
      // Up (38) key
      this.handleUpArrow(event);
    } else if (event.which === 40) {
      // Down (40) key press
      this.handleDownArrow(event);
    }
  }

  handleSelectItem(i) {
    this.commitSelection(i);
  }

  renderMenu() {
    if (!this.state.active) {
      return null;
    }

    const {
      items
    } = this.props;
    return /*#__PURE__*/React__default.createElement(Menu, null, items.map((item, i) => /*#__PURE__*/React__default.createElement(MenuItem, {
      item: item,
      index: i,
      onSelectItem: this.handleSelectItem,
      activateIndex: this.state.activateIndex,
      key: "item-" + i++
    })));
  }

  render() {
    const {
      inline,
      className
    } = this.props;
    return /*#__PURE__*/React__default.createElement(ClickableComponent, {
      className: classNames(className, {
        'cueplayer-react-menu-button-inline': !!inline,
        'cueplayer-react-menu-button-popup': !inline,
        'cueplayer-react-menu-button-active': this.state.active
      }, 'cueplayer-react-control cueplayer-react-button cueplayer-react-menu-button'),
      role: "button",
      tabIndex: "0",
      ref: c => {
        this.menuButton = c;
      },
      onClick: this.handleClick,
      onFocus: this.handleFocus,
      onBlur: this.handleBlur
    }, this.props.children, this.renderMenu());
  }

}
MenuButton.propTypes = propTypes$q;
MenuButton.displayName = 'MenuButton';

const propTypes$r = {
  player: PropTypes.object,
  actions: PropTypes.object,
  rates: PropTypes.array,
  className: PropTypes.string,
  onChange: PropTypes.func,
  selected: PropTypes.number
};
const defaultProps$7 = {
  rates: [2, 1.5, 1.25, 1, 0.5, 0.25]
};

class PlaybackRateMenuButton extends Component {
  constructor(props) {
    super(props);
    this.handleSelectItem = this.handleSelectItem.bind(this);
  }

  handleSelectItem(index) {
    const {
      rates,
      actions,
      onChange
    } = this.props;

    if (index >= 0 && index < rates.length) {
      actions.changeRate(rates[index]);

      if (onChange) {
        onChange(rates[index]);
      }
    }
  }

  render() {
    const {
      rates,
      player,
      selected
    } = this.props;
    const items = rates.map(rate => ({
      label: rate + "x",
      value: rate
    })); // remember 0  is a "false" in JS 🥴

    const selectedIndex = rates.indexOf(selected) > -1 ? rates.indexOf(selected) : rates.indexOf(player.playbackRate) || 0;
    return /*#__PURE__*/React__default.createElement(MenuButton, {
      className: classNames('cueplayer-react-playback-rate', this.props.className),
      onSelectItem: this.handleSelectItem,
      items: items,
      selectedIndex: selectedIndex
    }, /*#__PURE__*/React__default.createElement("span", {
      className: "cueplayer-react-control-text"
    }, "Playback Rate"), /*#__PURE__*/React__default.createElement("div", {
      className: "cueplayer-react-playback-rate-value"
    }, player.playbackRate.toFixed(2) + "x"));
  }

}

PlaybackRateMenuButton.propTypes = propTypes$r;
PlaybackRateMenuButton.defaultProps = defaultProps$7;
PlaybackRateMenuButton.displayName = 'PlaybackRateMenuButton';

const propTypes$s = {
  children: PropTypes.any,
  autoHide: PropTypes.bool,
  autoHideTime: PropTypes.number,
  // used in Player
  disableDefaultControls: PropTypes.bool,
  disableCompletely: PropTypes.bool,
  className: PropTypes.string
};
const defaultProps$8 = {
  autoHide: true,
  disableCompletely: false
};
class ControlBar extends Component {
  constructor(props) {
    super(props);
    this.getDefaultChildren = this.getDefaultChildren.bind(this);
    this.getFullChildren = this.getFullChildren.bind(this);
  }

  getDefaultChildren() {
    return [/*#__PURE__*/React__default.createElement(PlayToggle, {
      key: "play-toggle",
      order: 1
    }), /*#__PURE__*/React__default.createElement(VolumeMenuButton, {
      key: "volume-menu-button",
      order: 4
    }), /*#__PURE__*/React__default.createElement(CurrentTimeDisplay, {
      key: "current-time-display",
      order: 5.1
    }), /*#__PURE__*/React__default.createElement(TimeDivider, {
      key: "time-divider",
      order: 5.2
    }), /*#__PURE__*/React__default.createElement(DurationDisplay, {
      key: "duration-display",
      order: 5.3
    }), /*#__PURE__*/React__default.createElement(ProgressControl, {
      key: "progress-control",
      order: 6
    }), /*#__PURE__*/React__default.createElement(FullscreenToggle, {
      key: "fullscreen-toggle",
      order: 8
    })];
  }

  getFullChildren() {
    return [/*#__PURE__*/React__default.createElement(PlayToggle, {
      key: "play-toggle",
      order: 1
    }), /*#__PURE__*/React__default.createElement(ReplayControl, {
      key: "replay-control",
      order: 2
    }), /*#__PURE__*/React__default.createElement(ForwardControl, {
      key: "forward-control",
      order: 3
    }), /*#__PURE__*/React__default.createElement(VolumeMenuButton, {
      key: "volume-menu-button",
      order: 4
    }), /*#__PURE__*/React__default.createElement(CurrentTimeDisplay, {
      key: "current-time-display",
      order: 5
    }), /*#__PURE__*/React__default.createElement(TimeDivider, {
      key: "time-divider",
      order: 6
    }), /*#__PURE__*/React__default.createElement(DurationDisplay, {
      key: "duration-display",
      order: 7
    }), /*#__PURE__*/React__default.createElement(ProgressControl, {
      key: "progress-control",
      order: 8
    }), /*#__PURE__*/React__default.createElement(RemainingTimeDisplay, {
      key: "remaining-time-display",
      order: 9
    }), /*#__PURE__*/React__default.createElement(PlaybackRateMenuButton, {
      rates: [1, 1.25, 1.5, 2],
      key: "playback-rate",
      order: 10
    }), /*#__PURE__*/React__default.createElement(FullscreenToggle, {
      key: "fullscreen-toggle",
      order: 11
    })];
  }

  getChildren() {
    const children = React__default.Children.toArray(this.props.children);
    const defaultChildren = this.props.disableDefaultControls ? [] : this.getDefaultChildren();

    const _this$props = this.props,
          parentProps = _objectWithoutPropertiesLoose(_this$props, ["className"]); // remove className


    return mergeAndSortChildren(defaultChildren, children, parentProps);
  }

  render() {
    const {
      autoHide,
      className,
      disableCompletely
    } = this.props;
    const children = this.getChildren();
    return disableCompletely ? null : /*#__PURE__*/React__default.createElement("div", {
      className: classNames('cueplayer-react-control-bar', {
        'cueplayer-react-control-bar-auto-hide': autoHide
      }, className)
    }, children);
  }

}
ControlBar.propTypes = propTypes$s;
ControlBar.defaultProps = defaultProps$8;
ControlBar.displayName = 'ControlBar';

const USER_AGENT = typeof window !== 'undefined' && window.navigator ? window.navigator.userAgent : ''; // const webkitVersionMap = (/AppleWebKit\/([\d.]+)/i).exec(USER_AGENT);
// const appleWebkitVersion = webkitVersionMap ? parseFloat(webkitVersionMap.pop()) : null;

/*
 * Device is an iPhone
 *
 * @type {Boolean}
 * @constant
 * @private
 */

const IS_IPAD = /iPad/i.test(USER_AGENT); // The Facebook app's UIWebView identifies as both an iPhone and iPad, so
// to identify iPhones, we need to exclude iPads.
// http://artsy.github.io/blog/2012/10/18/the-perils-of-ios-user-agent-sniffing/

const IS_IPHONE = /iPhone/i.test(USER_AGENT) && !IS_IPAD;
const IS_IPOD = /iPod/i.test(USER_AGENT);
const IS_IOS = IS_IPHONE || IS_IPAD || IS_IPOD;

const defaultPlayerContext = {
  manager: {},
  setManager: () => {},
  getPlayer: () => initialState
};
const playerAtom = atom(initialState);
const PlayerContext = /*#__PURE__*/createContext(defaultPlayerContext);
function usePlayer() {
  return useContext(PlayerContext);
}
const PlayerProvider = (_ref) => {
  let {
    children
  } = _ref;
  const [manager, setManager] = useState();
  const [player, setPlayer] = useAtom(playerAtom);
  return /*#__PURE__*/createElement(PlayerContext.Provider, {
    value: {
      player,
      manager,
      setPlayer,
      setManager
    }
  }, children);
};

const propTypes$t = {
  children: PropTypes.any,
  width: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  height: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  fluid: PropTypes.bool,
  muted: PropTypes.bool,
  playsInline: PropTypes.bool,
  aspectRatio: PropTypes.string,
  className: PropTypes.string,
  videoId: PropTypes.string,
  startTime: PropTypes.number,
  loop: PropTypes.bool,
  autoPlay: PropTypes.bool,
  src: PropTypes.string,
  poster: PropTypes.string,
  preload: PropTypes.oneOf(['auto', 'metadata', 'none']),
  onLoadStart: PropTypes.func,
  onWaiting: PropTypes.func,
  onCanPlay: PropTypes.func,
  onCanPlayThrough: PropTypes.func,
  onPlaying: PropTypes.func,
  onEnded: PropTypes.func,
  onSeeking: PropTypes.func,
  onSeeked: PropTypes.func,
  onPlay: PropTypes.func,
  onPause: PropTypes.func,
  onProgress: PropTypes.func,
  onDurationChange: PropTypes.func,
  onError: PropTypes.func,
  onSuspend: PropTypes.func,
  onAbort: PropTypes.func,
  onEmptied: PropTypes.func,
  onStalled: PropTypes.func,
  onLoadedMetadata: PropTypes.func,
  onLoadedData: PropTypes.func,
  onTimeUpdate: PropTypes.func,
  onRateChange: PropTypes.func,
  onVolumeChange: PropTypes.func,
  store: PropTypes.object
};
const defaultProps$9 = {
  fluid: true,
  muted: false,
  playsInline: false,
  preload: 'auto',
  aspectRatio: 'auto'
};
class Player extends Component {
  constructor(props) {
    super(props);
    this.controlsHideTimer = null;
    this.video = null; // the Video component

    this.manager = new Manager(props.store);
    this.actions = this.manager.getActions();
    this.manager.subscribeToPlayerStateChange(this.handleStateChange.bind(this));
    this.getStyle = this.getStyle.bind(this);
    this.handleResize = this.handleResize.bind(this);
    this.getChildren = this.getChildren.bind(this);
    this.handleMouseMove = throttle(this.handleMouseMove.bind(this), 250);
    this.handleMouseDown = this.handleMouseDown.bind(this);
    this.startControlsTimer = this.startControlsTimer.bind(this);
    this.handleFullScreenChange = this.handleFullScreenChange.bind(this);
    this.handleKeyDown = this.handleKeyDown.bind(this);
    this.handleFocus = this.handleFocus.bind(this);
    this.handleBlur = this.handleBlur.bind(this);
  }

  componentDidMount() {
    var _this$context;

    this.handleResize();
    window.addEventListener('resize', this.handleResize);
    fullscreen.addEventListener(this.handleFullScreenChange);
    this.getMetadataTextTracks();

    if ((_this$context = this.context) != null && _this$context.setManager) {
      this.context.setManager(this.manager);
    }
  }

  componentDidUpdate() {
    var _this$context2;

    this.getMetadataTextTracks();

    if ((_this$context2 = this.context) != null && _this$context2.setPlayer) {
      const {
        player
      } = this.manager.getState();
      this.context.setPlayer(player);
    }
  }

  componentWillUnmount() {
    // Remove event listener
    window.removeEventListener('resize', this.handleResize);
    fullscreen.removeEventListener(this.handleFullScreenChange);

    if (this.controlsHideTimer) {
      window.clearTimeout(this.controlsHideTimer);
    }
  }

  getDefaultChildren(originalChildren) {
    return [/*#__PURE__*/React__default.createElement(Video, {
      ref: c => {
        this.video = c;
        this.manager.video = this.video;
      },
      key: "video",
      order: 0.0
    }, originalChildren), /*#__PURE__*/React__default.createElement(PosterImage, {
      key: "poster-image",
      order: 1.0
    }), /*#__PURE__*/React__default.createElement(LoadingSpinner, {
      key: "loading-spinner",
      order: 2.0
    }), /*#__PURE__*/React__default.createElement(Bezel, {
      key: "bezel",
      order: 3.0
    }), /*#__PURE__*/React__default.createElement(BigPlayButton, {
      key: "big-play-button",
      order: 4.0
    }), /*#__PURE__*/React__default.createElement(ControlBar, {
      key: "control-bar",
      order: 5.0
    }), /*#__PURE__*/React__default.createElement(Shortcut, {
      key: "shortcut",
      order: 99.0
    })];
  }

  getChildren(props) {
    const {
      // remove className
      children: originalChildren
    } = props,
          propsWithoutChildren = _objectWithoutPropertiesLoose(props, ["className", "children"]);

    const children = React__default.Children.toArray(this.props.children).filter(e => !isVideoChild(e));
    const defaultChildren = this.getDefaultChildren(originalChildren);
    return mergeAndSortChildren(defaultChildren, children, propsWithoutChildren);
  }

  setWidthOrHeight(style, name, value) {
    let styleVal;

    if (typeof value === 'string') {
      if (value === 'auto') {
        styleVal = 'auto';
      } else if (value.match(/\d+%/)) {
        styleVal = value;
      }
    } else if (typeof value === 'number') {
      styleVal = value + "px";
    }

    Object.assign(style, {
      [name]: styleVal
    });
  }

  getStyle() {
    const {
      fluid,
      aspectRatio: propsAspectRatio,
      height: propsHeight,
      width: propsWidth
    } = this.props;
    const {
      player
    } = this.manager.getState();
    const style = {};
    let width;
    let height;
    let aspectRatio; // The aspect ratio is either used directly or to calculate width and height.

    if (propsAspectRatio !== undefined && propsAspectRatio !== 'auto') {
      // Use any aspectRatio that's been specifically set
      aspectRatio = propsAspectRatio;
    } else if (player.videoWidth) {
      // Otherwise try to get the aspect ratio from the video metadata
      aspectRatio = player.videoWidth + ":" + player.videoHeight;
    } else {
      // Or use a default. The video element's is 2:1, but 16:9 is more common.
      aspectRatio = '16:9';
    } // Get the ratio as a decimal we can use to calculate dimensions


    const ratioParts = aspectRatio.split(':');
    const ratioMultiplier = ratioParts[1] / ratioParts[0];

    if (propsWidth !== undefined) {
      // Use any width that's been specifically set
      width = propsWidth;
    } else if (propsHeight !== undefined) {
      // Or calulate the width from the aspect ratio if a height has been set
      width = propsHeight / ratioMultiplier;
    } else {
      // Or use the video's metadata, or use the video el's default of 300
      width = player.videoWidth || 400;
    }

    if (propsHeight !== undefined) {
      // Use any height that's been specifically set
      height = propsHeight;
    } else {
      // Otherwise calculate the height from the ratio and the width
      height = width * ratioMultiplier;
    }

    if (fluid) {
      style.paddingTop = ratioMultiplier * 100 + "%";
    } else {
      // If Width contains "auto", set "auto" in style
      this.setWidthOrHeight(style, 'width', width);
      this.setWidthOrHeight(style, 'height', height);
    }

    return style;
  } // get redux state
  // { player, operation }


  getState() {
    return this.manager.getState();
  } // get playback rate


  get playbackRate() {
    return this.video.playbackRate;
  } // set playback rate
  // speed of video


  set playbackRate(rate) {
    this.video.playbackRate = rate;
  }

  get muted() {
    return this.video.muted;
  }

  set muted(val) {
    this.video.muted = val;
  }

  get volume() {
    return this.video.volume;
  }

  set volume(val) {
    this.video.volume = val;
  } // video width


  get videoWidth() {
    return this.video.videoWidth;
  } // video height


  get videoHeight() {
    return this.video.videoHeight;
  }

  getMetadataTextTracks() {
    const {
      player
    } = this.manager.getState();
    const {
      textTracks
    } = player;
    const tracks = Array.from(textTracks || []).filter(track => {
      return ['metadata'].includes(track.kind);
    });
    tracks.forEach(textTrack => {
      if (textTrack.mode !== 'showing') {
        this.actions.activateMetadataTrack(textTrack);
      }
    });
    return tracks;
  } // play the video


  play() {
    this.video.play();
  } // pause the video


  pause() {
    this.video.pause();
  } // Change the video source and re-load the video:


  load() {
    this.video.load();
  } // Add a new text track to the video


  addTextTrack() {
    this.video.addTextTrack(...arguments);
  } // Check if your browser can play different types of video:


  canPlayType() {
    this.video.canPlayType(...arguments);
  } // seek video by time


  seek(time) {
    this.video.seek(time);
  } // jump forward x seconds


  forward(seconds) {
    this.video.forward(seconds);
  } // jump back x seconds


  replay(seconds) {
    this.video.replay(seconds);
  } // enter or exist full screen


  toggleFullscreen() {
    this.video.toggleFullscreen();
  } // subscribe to player state change


  subscribeToStateChange(listener) {
    return this.manager.subscribeToPlayerStateChange(listener);
  } // player resize


  handleResize() {}

  handleFullScreenChange(event) {
    if (event.target === this.manager.rootElement) {
      this.actions.handleFullscreenChange(fullscreen.isFullscreen);
    }
  }

  handleMouseDown() {
    this.startControlsTimer();
  }

  handleMouseMove() {
    this.startControlsTimer();
  }

  handleKeyDown() {
    this.startControlsTimer();
  }

  startControlsTimer() {
    let controlBarActiveTime = 3000;
    React__default.Children.forEach(this.props.children, element => {
      if (! /*#__PURE__*/React__default.isValidElement(element) || element.type !== ControlBar) {
        return;
      }

      const {
        autoHideTime
      } = element.props;

      if (typeof autoHideTime === 'number') {
        controlBarActiveTime = autoHideTime;
      }
    });
    this.actions.userActivate(true);
    clearTimeout(this.controlsHideTimer);
    this.controlsHideTimer = setTimeout(() => {
      this.actions.userActivate(false);
    }, controlBarActiveTime);
  }

  handleStateChange(state, prevState) {
    if (state.isFullscreen !== prevState.isFullscreen) {
      this.handleResize(); // focus root when switching fullscreen mode to avoid confusion #276

      focusNode(this.manager.rootElement);
    }

    this.forceUpdate(); // re-render
  }

  handleFocus() {
    this.actions.activate(true);
  }

  handleBlur() {
    this.actions.activate(false);
  }

  render() {
    const {
      fluid
    } = this.props;
    const {
      player
    } = this.manager.getState();
    const {
      paused,
      hasStarted,
      waiting,
      seeking,
      isFullscreen,
      userActivity,
      activeMetadataTracks
    } = player; // all children get all props

    const props = _extends({}, this.props, {
      player,
      actions: this.actions,
      manager: this.manager,
      store: this.manager.store,
      video: this.video ? this.video.video : null
    });

    const children = this.getChildren(props);
    return /*#__PURE__*/React__default.createElement("div", {
      className: classNames({
        'cueplayer-react-controls-enabled': true,
        'cueplayer-react-has-started': hasStarted,
        'cueplayer-react-paused': paused,
        'cueplayer-react-playing': !paused,
        'cueplayer-react-waiting': waiting,
        'cueplayer-react-seeking': seeking,
        'cueplayer-react-fluid': fluid,
        'cueplayer-react-fullscreen': isFullscreen,
        'cueplayer-react-user-inactive': !userActivity,
        'cueplayer-react-user-active': userActivity,
        'cueplayer-react-workinghover': !IS_IOS,
        'cueplayer-react-cues-active': !isEmpty(activeMetadataTracks)
      }, 'cueplayer-react', this.props.className),
      style: this.getStyle(),
      ref: c => {
        this.manager.rootElement = c;
      },
      role: "region",
      onTouchStart: this.handleMouseDown,
      onMouseDown: this.handleMouseDown,
      onTouchMove: this.handleMouseMove,
      onMouseMove: this.handleMouseMove,
      onKeyDown: this.handleKeyDown,
      onFocus: this.handleFocus,
      onBlur: this.handleBlur,
      tabIndex: "-1"
    }, children);
  }

}
Player.contextType = PlayerContext;
Player.propTypes = propTypes$t;
Player.defaultProps = defaultProps$9;
Player.displayName = 'Player';

class PlaybackRate extends Component {
  constructor(props) {
    super(props);
    deprecatedWarning('PlaybackRate', 'PlaybackRateMenuButton');
  }

  render() {
    return /*#__PURE__*/React__default.createElement(PlaybackRateMenuButton, this.props);
  }

}
PlaybackRate.displayName = 'PlaybackRate';

const propTypes$u = {
  player: PropTypes.object,
  actions: PropTypes.object,
  className: PropTypes.string,
  offMenuText: PropTypes.string,
  showOffMenu: PropTypes.bool,
  kinds: PropTypes.array
};
const defaultProps$a = {
  offMenuText: 'Off',
  showOffMenu: true,
  kinds: ['captions', 'subtitles'] // `kind`s of TextTrack to look for to associate it with this menu.

};

class ClosedCaptionButton extends Component {
  constructor(props) {
    super(props);
    this.getTextTrackItems = this.getTextTrackItems.bind(this);
    this.updateState = this.updateState.bind(this);
    this.handleSelectItem = this.handleSelectItem.bind(this);
    this.state = this.getTextTrackItems();
  }

  componentDidMount() {
    const {
      selected,
      player,
      actions
    } = this.props;
    const {
      textTracks = []
    } = player;

    if (selected && textTracks.length > 0) {
      Array.from(textTracks).forEach(textTrack => {
        if (textTrack.language === selected.language) {
          textTrack.mode = 'showing';
          actions.activateTextTrack(textTrack);
        }
      });
    }
  }

  componentDidUpdate() {
    this.updateState();
  }

  getTextTrackItems() {
    const {
      kinds,
      player,
      offMenuText,
      showOffMenu
    } = this.props;
    const {
      textTracks,
      activeTextTrack
    } = player;
    const textTrackItems = {
      items: [],
      selectedIndex: 0
    };
    const tracks = Array.from(textTracks || []);

    if (tracks.length === 0) {
      return textTrackItems;
    }

    if (showOffMenu) {
      textTrackItems.items.push({
        label: offMenuText || 'Off',
        value: null
      });
    }

    tracks.forEach(textTrack => {
      // ignore invalid text track kind
      if (kinds.length && !kinds.includes(textTrack.kind)) {
        return;
      }

      textTrackItems.items.push({
        label: textTrack.label,
        value: textTrack.language
      });
    });
    textTrackItems.selectedIndex = textTrackItems.items.findIndex(item => activeTextTrack && activeTextTrack.language === item.value);

    if (textTrackItems.selectedIndex === -1) {
      textTrackItems.selectedIndex = 0;
    }

    return textTrackItems;
  }

  updateState() {
    const textTrackItems = this.getTextTrackItems();

    if (textTrackItems.selectedIndex !== this.state.selectedIndex || !this.textTrackItemsAreEqual(textTrackItems.items, this.state.items)) {
      this.setState(textTrackItems);
    }
  }

  textTrackItemsAreEqual(items1, items2) {
    if (items1.length !== items2.length) {
      return false;
    }

    for (let i = 0; i < items1.length; i++) {
      if (!items2[i] || items1[i].label !== items2[i].label || items1[i].value !== items2[i].value) {
        return false;
      }
    }

    return true;
  }

  handleSelectItem(index) {
    const {
      player,
      actions,
      showOffMenu,
      onChange
    } = this.props;
    const {
      textTracks
    } = player;
    let selectedTrack; // For the 'subtitles-off' button, the first condition will never match
    // so all subtitles will be turned off

    Array.from(textTracks).forEach((textTrack, i) => {
      // if it shows the `Off` menu, the first item is `Off`
      if (index === (showOffMenu ? i + 1 : i)) {
        textTrack.mode = 'showing';
        selectedTrack = textTrack;
        actions.activateTextTrack(selectedTrack);
      } else {
        textTrack.mode = 'hidden';
      }
    });

    if (onChange) {
      onChange(selectedTrack);
    }
  }

  render() {
    const {
      items,
      selectedIndex
    } = this.state;
    return /*#__PURE__*/React__default.createElement(MenuButton, {
      className: classNames('cueplayer-react-closed-caption', this.props.className),
      onSelectItem: this.handleSelectItem,
      items: items,
      selectedIndex: selectedIndex
    }, /*#__PURE__*/React__default.createElement("span", {
      className: "cueplayer-react-control-text"
    }, "Closed Caption"));
  }

}

ClosedCaptionButton.propTypes = propTypes$u;
ClosedCaptionButton.defaultProps = defaultProps$a;
ClosedCaptionButton.displayName = 'ClosedCaptionButton';

export { Bezel, BigPlayButton, ClosedCaptionButton, ControlBar, CurrentTimeDisplay, DurationDisplay, ForwardControl, FullscreenToggle, LoadProgressBar, LoadingSpinner, MenuButton, MouseTimeDisplay, PlayProgressBar, PlayToggle, PlaybackRate, PlaybackRateMenuButton, Player, PlayerContext, PlayerProvider, PosterImage, ProgressControl, RemainingTimeDisplay, ReplayControl, SeekBar, Shortcut, Slider, TimeDivider, Video, VolumeMenuButton, fullscreen, operationReducer, playerActions, playerAtom, playerReducer, usePlayer, videoActions };
//# sourceMappingURL=cueplayer-react.es.js.map
